#!usr/bin/perl

#Cluster subroutine required for Norwegian decomposition tool

##########################################################################################
#Decopmpose on the basis of consonant clusters

sub cluster {
   my $wd = shift;
   my $orig = $wd;

   print LOGG "Entered sub cluster with $wd\n" if $lg;

   return $wd if ($wd !~ /$lc/);		#*LO-lederen

#this used to be here, but moved down to after prefix/suffix strip
#   foreach $key (keys (%not_cmp)) {
#      return $wd if ($wd =~ /^$key(sk|isering)?(~s)?$/);	# don't do anything if item is a recognised non-compound (tekstil, luxembourgsk)
#   }

   #general word-final replacement (undervann~s+skrog)
   $wd =~ s/($db_con)s$/$1~s/;

   #Find relevant cluster
   #make list of relevant clusters in word, ie word-medial ones (after ending strip)
   $tmp = $wd;
   $tmp =~ s/^(u)?($prefix)// if ($tmp !~ /^(avls|be)/);	# get rid of possible initial prefix; added 15/11
   $tmp =~ s/($suffix)(~s)?$//;	# get rid of possible final suffix; added 15/11

   foreach $key (keys (%not_cmp)) {
      return $wd if ($tmp =~ /^$key(sk|isering|a|et|ete)?$/);	# don't do anything if item is a recognised non-compound (tekstil, luxembourgsk)
   }

   $tmp =~ s/^($con)+//;	# get rid of possible initial consonants
   $tmp =~ s/^($vow)+//;	# get rid of possible initial vowels
   $tmp =~ s/($con)+$//;	# get rid of possible final consonants
   $tmp =~ s/($vow)+$//;	# get rid of possible final vowels
   #all remaining clusters should be analysed
   $tmp =~ s/($vow)+//g;

   @relevant_clusters = split "", $tmp;
   foreach $cl (@relevant_clusters) {
      $ocl = $cl;

      next if ($cl =~ /[\+~]/);
      next if ($cl !~ /../);		# only 2+-letter clusters are handled here

      print LOGG "cluster: $cl\n";

#HERE YOU START THE REAL CLUSTER RULES

#BY INITIAL CONSONANTS
#      if ($cl =~ /^p/) {$cl = &p($cl)};
#      if ($cl =~ /^n/) {$cl = &n_cluster($cl,$wd)};


#BY NUMBER OF CHARACTERS
	#7
	if ($cl =~ /^.......$/) {				# angst+skrik
		foreach $key (keys (%where_7)) {
			last if ($cl =~ s/$key/$where_7{$key}/);
		}
		$cl =~ s/($con)s\+/$1\~s\+/;
		print LOGG "AFTER 7 REPLACEMENT, cluster: $cl\n";
	}

	#6
	elsif ($cl =~ /^......$/) {				# hy+trykk~s+spryte
		foreach $key (keys (%where_6)) {
			last if ($cl =~ s/$key/$where_6{$key}/); # use last or not?
		}
		$cl =~ s/($con)s\+/$1\~s\+/;
		print LOGG "AFTER 6 REPLACEMENT, cluster: $cl\n";
	}
	#5
	elsif ($cl =~ /^.....$/) {				#
		foreach $key (keys (%where_5)) {
			last if ($cl =~ s/$key/$where_5{$key}/);
		}
		$cl =~ s/($con)s\+/$1\~s\+/;
		print LOGG "AFTER 5 REPLACEMENT, cluster: $cl\n";
	}
	#4
	elsif ($cl =~ /^....$/) {				#
		if (($cl =~ /.str$/) & ($wd =~ /...str/)) { 
			if ($wd =~ /$str_initial/) {$cl =~ s/str/\+str/;}
			elsif ($wd =~ /(venstr|m(o|)nstr|blomstr|instru|konstruk|demonstr|ekstr(a|e))/) {next;}
			else {$cl =~ s/str/~s\+tr/;}
		}
		elsif (($cl =~ /.stj$/) & ($wd =~ /...stj/)) { 
			if ($wd =~ /...$stj_initial/) {$cl =~ s/stj/\+stj/;}
			else {$cl =~ s/stj/~s\+tj/;}
		}
		elsif (($cl =~ /[^s]spr$/) & ($wd =~ /...spr/)) { 
			if ($wd =~ /...$spr_initial/) {$cl =~ s/spr/\+spr/;}
			elsif ($wd =~ /(ekspre)/) {next;}
			else {$cl =~ s/spr/~s\+pr/;}
		}
		elsif (($cl =~ /.spl$/) & ($wd =~ /...spl/)) { 
			if ($wd =~ /...$spl_initial/) {$cl =~ s/spl/\+spl/;}
			elsif ($wd =~ /(ekspli)/) {next;}
			else {$cl =~ s/spl/~s\+pl/;}
		}
		elsif (($cl =~ /.skr$/) & ($wd =~ /...skr/)) { 
			if ($wd =~ /...$skr_initial/) {$cl =~ s/skr/\+skr/;}
			elsif ($wd =~ /(skribe|skrip)/) {next;}
			else {$cl =~ s/skr/~s\+kr/;}
		}
		elsif (($cl =~ /.spj$/) & ($wd =~ /...spj/)) { 
			if ($wd =~ /...$spj_initial/) {$cl =~ s/spj/\+spj/;}
			else {$cl =~ s/spj/~s\+pj/;}
		}
		elsif (($cl =~ /.skv$/) & ($wd =~ /...skv/)) { 
			if ($wd =~ /...$skv_initial/) {$cl =~ s/skv/\+skv/;}
			else {$cl =~ s/skv/~s\+kv/;}
		}
		elsif (($cl =~ /.skj$/) & ($wd =~ /...skj/)) { 
			if ($wd =~ /...$skj_initial/) {$cl =~ s/skj/\+skj/;}
			else {$cl =~ s/skj/~s\+kj/;}
		}
		
		#Various 4-con exceptions
#		elsif ((($cl =~ /..sj/) & ($tmp =~ /sjon/)) |
#			(($cl =~ /.skl/) & ($tmp =~ /eksklu/)) |
#			(($cl =~ /.spr/) & ($tmp =~ /ekspre/))
#			) {print LOGG "HEI OG H!!!\n"; next;}

		#Double consonant + 2 cons
#		elsif ($cl =~ /^($db_con)[^s].$/) {
#			$cl =~ s/($db_con)/$1\+/;
#		}

		#ng + 2 cons must generally be ng+.. I guess (gang+sperr), since ~ing~s has been taken care of before.
#		elsif ($cl =~ /^ng..$/) {
#			$cl =~ s/ng/ng\+/;
#		}

		else {
			foreach $key (keys (%where_4)) {
				last if ($cl =~ s/$key/$where_4{$key}/);
			}
			$cl =~ s/($con_no_s)s\+/$1\~s\+/;
		}

		print LOGG "AFTER 4 REPLACEMENT, cluster: $cl\n";
	}
	#3 consonants, first check doubles
	#usually decompose, but not in akkreditering, lavinntekstsgruppe

	elsif (($cl =~ /^($db_con)($con)$/) & ($cl !~ /(kkr|nnt)/)) {
		$cl =~ s/($db_con)/$1\+/;
	}

	elsif ($cl =~ /^...$/) {				#

		foreach $key (keys (%where_3)) {
			$cl =~ s/$key/$where_3{$key}/;
		}
		$cl =~ s/($con_no_s)s\+/$1\~s\+/;
		print LOGG "AFTER 3 REPLACEMENT, cluster: $cl\n";
	}

	# 2 consonants, you'd better be careful...
	elsif ($cl =~ /^..$/) {				#

	#2-con clusters that virtually always (???check!) involve decomposition
	# if not, word must be added in not_compound
#		if ($cl !~ /($db_con|$legal|($con)s)/) {
		if (($cl =~ /(($con)h|b(k|s)|d(b|c|f|k|p|v)|f(b|d|j|k)|g(b|c|d|f|j|k|m|p|v)|k(b|d|f|g|h|m|p))/) or 
		    ($cl =~ /(l(b|c|h|r)|m(b|c|d|f|g|h|k|v)|n(b|c|f|h|m|p|r|v)|p(d|f|g|h|k|m|n|v))/) or
		    ($cl =~ /(r(c|f|h)|s(b|d|f|g|h|r)|t(b|c|d|f|g|h|k|p)|v(b|c|f|g|k|m|p|s|v)|wm|x($con))/)) {
			$cl =~ s/^(.)(.)$/$1\+$2/;
		}
		print LOGG "AFTER 2 REPLACEMENT, cluster: $cl\n";
	}

      $wd =~ s/$ocl/$cl/;

   }

   print LOGG "$wd before return cluster\n" if $lg;
   return $wd;

} #end sub cluster
##########################################################################################

#Decopmpose word containing compound-final vow + con

sub vow_con {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub VOW_CON with $wd\n" if $lg;

	foreach $key (keys (%ends_in_vow_con)) {
		if ($wd =~ /($key).../) {
			$wd = &split_or_not_after_key($wd,$key);
		}
	}

	print LOGG "$wd after checked ends_in_vow_con\n" if $lg;

	return $wd;

} #end sub vow_con
##########################################################################################

#Decopmpose word containing compound-initial con + vow

sub con_vow {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub CON_VOW with $wd\n" if $lg;

	foreach $key (keys (%begs_with_con_vow)) {
		if ($wd =~ /...($key)/) {
			$wd = &split_or_not_before_key($wd,$key);
		}
	}

	print LOGG "$wd after checked begs_with_con_vow\n" if $lg;

	return $wd;

} #end sub con_vow
##########################################################################################

1;
