#!usr/bin/perl

#Subroutines required for Norwegian decomposition tool

##########################################################################################
#Strip punctuation

sub punctuation {
	my $wd = shift;
	my $orig = $wd;

	$wd =~ s/[\.\,\:\;\!\?\"\'\(\)\{\}\[\]]//g;

	print LOGG "$wd before return punctuation\n" if $lg;
	return $wd;
} #end sub



##########################################################################################
#Strip punctuation

sub case {
	my $wd = shift;
	my $orig = $wd;

	if ($wd =~ /^($uc)($lc)/) {
		$init_cap = 1;
		$wd = lc $wd;
	}

	print LOGG "$wd before return case\n" if $lg;
	return $wd;
} #end sub


##########################################################################################
#Strip inflections

sub inflection {
	my $wd = shift;
	my $orig = $wd;

	return $wd if $wd =~ /($db_con)$/;
	$wd =~ s/(...)($inflection)$/$1~$2/;

	print LOGG "$wd before return inflection\n" if $lg;
	return $wd;
} #end sub

##########################################################################################
#Handle hyphenated compounds

sub hyphen {
	my $wd = shift;
	my $orig = $wd;
	
	return $wd if $wd !~ /\-/;
	
	$wd =~ s/\-/\+\-\+/g;

	print LOGG "$wd before return hyphen\n" if $lg;
	return $wd;
} #end sub hyphen

##########################################################################################
#Handle geminates like busstopp -> buss+stopp
#Match if matches from beginning to end. 

sub geminate_compound {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub geminate_compound with $wd\n" if $lg;
	
	foreach $key (keys (%gem_cmp)) {
		if ($wd =~ s/^$key$/$gem_cmp{$key}/) {return $wd};	# buss+stopp
	}

	print LOGG "$wd before return geminate_compound\n" if $lg;

	return $wd;
} #end sub 

##########################################################################################
#Handle exceptions
#Match if matches from beginning. 

sub full_compound {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub full_compound with $wd\n" if $lg;
	
	foreach $key (keys (%full_cmp)) {
		$compare = $key;
		$compare =~ s/[\+\~]//g;
		if ($wd =~ s/^($compare)(\~(s|e)?)?/$key$2/) {return $wd};		# adam+stua
		if ($wd =~ s/($compare)$/$key/) {return $wd};		# anleggsreserve
	}

	print LOGG "$wd before return full_compound\n" if $lg;

	return $wd;
} #end sub 

##########################################################################################

#Handle words with rare link morphemes
#Match if matches from beginning to end. 

sub rare_link_morph {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub rare_link_morph with $wd\n" if $lg;
	
	foreach $key (keys (%er_cmp)) {
		if ($wd =~ /^$key$/) {
			$wd =~ s/(..)er(..)/$1~er\+$2/;		# lur~en+dreier
		return $wd;
		}
	}

	foreach $key (keys (%a_cmp)) {
		if ($wd =~ /^$key$/) {
			$wd =~ s/(..)a(..)/$1~a\+$2/;		# vest~a+vind
		return $wd;
		}
	}

	foreach $key (keys (%en_cmp)) {
		if ($wd =~ /^$key$/) {
			$wd =~ s/(..)en(..)/$1~en\+$2/;		# task~en+spiller
		}
	}

	print LOGG "$wd before return rare_link_morph\n" if $lg;
	return $wd;
} #end sub 

##########################################################################################

#Handle short compounds
#Match if matches from beginning to end. 

sub short_compound {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub short_compound with $wd\n" if $lg;
	
	foreach $key (keys (%short_cmp)) {
		$compare = $key;
		$compare =~ s/\+//g;
		$wd =~ s/^($compare)(\~(s|e)?)$/$key$2/;		# adam+stua
	}

	print LOGG "$wd before return short_compound\n" if $lg;
	return $wd;
} #end sub 

##########################################################################################
#Handle compounds with common derivational suffixes

sub pfx_sfx {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub pfx_sfx with $wd\n" if $lg;

	foreach $k (keys (%not_cmp)) {
#18/6-03, narrowing this to full match, other
		return $wd if ($k =~ /^$wd/);	# don't if item is a recognised non-compound
	}

print LOGG "AT 1!:\t $wd\n";
#SOME EXCEPTIONS FIRST perhaps? eksperiment+stadium, assistent+stilling
	return $wd if /somhet$/;
	return $wd if /beaufort/;
	if ($wd =~ s/(...)(ment|ent)(st)/$1$2\+$3/) {print LOGG "RETURNING AT 1\n"; return $wd;};
	if ($wd =~ s/(...)(fisk|frisk|stilling|befordr)/$1\+$2/) {print LOGG "RETURNING AT 2\n"; return $wd;};
	if ($wd =~ s/(ing)(spill|spell|steng)/$1\+$2/) {print LOGG "RETURNING AT 3\n"; return $wd;};
	if ($wd =~ s/(..)([^s])(innstill|selskap)/$1$2\+$3/) {print LOGG "RETURNING AT 4\n"; return $wd;};

	$wd =~ s/(...)($suffix)s(...)/$1$2~s\+$3/;		# kristendom~s+kunnskap
	$wd =~ s/(...)($suffix_u_s)(...)/$1$2\+$3/ unless ($wd =~ /($suffix|$suffix_u_s)($suffix|$suffix_u_s)/);		# walisisk+liknende
print LOGG "AT 2!:\t $wd\n";
	$wd =~ s/(..)($con_no_s)s($prefix)(...)/$1$2~s\+$3$4/ if {$wd !~ /(^bestill|innstill)|(somhet$)/};	# lnn~s+oppgjr
print LOGG "AT 3!:\t $wd\n";
	$wd =~ s/(...)e($prefix)(...)/$1~e\+$2$3/ if {$wd !~ /^bestilling/};	# lnn~s+oppgjr
print LOGG "AT 4!:\t $wd\n";
	$wd =~ s/(...)($prefix)(...)/$1\+$2$3/ unless {$wd =~ /^bestilling/};	# fregatt+oppdrag
print LOGG "AT 5!:\t $wd\n";
	$wd =~ s/(...)(for)(...)/$1\+$2$3/; 
	$wd =~ s/(...)(ist|ikk)(opp)(...)/$1$2\+$3$4/; 
print LOGG "AT 6!:\t $wd\n";
	print LOGG "$wd at middle of pfx_sfx\n" if $lg;
	
	$wd =~ s/\+\+/\+/g;
	$wd =~ s/\+~/\+/g;
	$wd =~ s/~s\+s\+/~s\+s/g;

#print LOGG "HERE 5!\n";

	print LOGG "$wd before return pfx_sfx\n" if $lg;
	return $wd;
} #end sub pfx_sfx

##########################################################################################
#Handle compounds with common rightmost component

sub common_rmcc {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered common_rmcc with $wd\n" if $lg;

	#Testing for rightmost component
	foreach $key (reverse sort keys (%cm_rmcc)) {
		if ($wd =~ /...($key)/) {
			$wd = &split_or_not_before_key($wd,$key);
			last if $wd != $orig;
		}
	}

	print LOGG "$wd before return common_rmcc\n" if $lg;
	return $wd;
} #end sub rmcc

##########################################################################################

#Handle compounds with common leftmost component

sub common_lmcc {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered common_lmcc with $wd\n" if $lg;

	#Testing for leftmost component
	foreach $key (reverse sort keys (%cm_lmcc)) {
		if ($wd =~ /($key).../) {
			$wd = &split_or_not_after_key($wd,$key);
			last if $wd != $orig;
			last if $wd =~ /blekk\+slager/;
		}
	}

	print LOGG "$wd before return common_lmcc\n" if $lg;
	return $wd;
} #end sub rmcc

##########################################################################################

#Check if e should be stripped or not

sub check_e_strip {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub check_e_strip with $wd\n" if $lg;
	return $wd if $wd !~ /~e\+/;
	
	foreach $key (keys (%e_strip)) {
		if ($wd =~ /(^|\+)$key~e/) {
			return $wd;
		}
	}

	$wd =~ s/~e/e/g;

	print LOGG "$wd before return check_e_strip\n" if $lg;
	return $wd;
} #end sub 

##########################################################################################

#Check if s should be stripped or not

sub check_s_strip {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered sub check_s_strip with $wd\n" if $lg;
	return $wd if $wd !~ /~s/;
	
	if ($wd =~ /(^|\+)($no_s_strip)~s/) {
		$wd =~ s/\~s$/s/;
		print LOGG "Decided not to strip s in $wd\n" if $lg;
		return $wd;
	}
	else {
	
		print LOGG "$wd before return check_s_strip\n" if $lg;
		return $wd;
	}
} #end sub 

##########################################################################################

#Analyses relationship between $wd and $key and determines whether to decompose after key or not.

sub split_or_not_after_key {
	my $wd = shift;
	my $key = shift;
	my $orig = $wd;

	print LOGG " Entered sub SPLIT_OR_NOT_AFTER_KEY with word $wd and key $key\n" if $lg;

	@rest = split /$key/, $wd;
	$rest_bef_key = $rest[0];
	$rest_aft_key = $rest[1];
	$left_and_key = $rest_bef_key.$key;
	$key_and_right = $key.$rest_aft_key;

	print LOGG "If I decide to split, remnant \$rest_aft_key will be $rest_aft_key after key \n" if $lg;
	print LOGG "If I decide to split, remnant \$rest_bef_key will be $rest_bef_key before key \n" if $lg;
	print LOGG "If I decide to split, variable \$left_and_key will be $left_and_key before key \n" if $lg;
	print LOGG "If I decide to split, variable \$key_and_right will be $key_and_right after key \n" if $lg;

	# Conditions for failure to split after key
	# don't if item is a recognised non-compound
	# that is, if left remnant and key match item in not_compound
	foreach $k (keys (%not_cmp)) {
		return $wd if (($left_and_key =~ /^$k/) & ($key != $k));
#		return $wd if ($left_and_key =~ /^$k/);
		return $wd if ($key_and_right =~ /^(s)?$k/);
	}
		print LOGG "GOT TO 1\n";
	return $wd if ($rest_bef_key =~ /^.$/); 	# dont split fri in afrikaner
	return $wd if ($rest_bef_key =~ /^..$/); 	# dont split topp in altoppslukende
		print LOGG "GOT TO 2\n";
	return $wd if (($key =~ /s$/) & ($rest_aft_key =~ /^(jon|kap)/));
	return $wd if (($key =~ /e$/) & ($rest_aft_key =~ /^ring/)); #*adressering
		print LOGG "GOT TO 3\n";
	return $wd if ($key_and_right =~ /ordring/); 	# befordrning
	return $wd if ($rest_aft_key =~ /^($suffix|(is|ul)?ering|ging|ist|elig)/); 	# *skur+ing, *akkreditering
	return $wd if ($rest_aft_key =~ /^($con)er$/); 	# avstandsmler
		print LOGG "GOT TO 4\n";
	return $wd if ($rest_bef_key =~ /($prefix)$/); 	#
		print LOGG "GOT TO 5\n";

	if ((($wd =~ /$key($lc{3})/)) &	# don't if not followed by three characters
	    ($rest_aft_key =~ /^(($init)($vow)|(($con)($vow))|($vow))/) & # don't if rest doesn't begin with legal initial cluster
	    (($rest_bef_key == "") | ($rest_bef_key =~ /(($vow)($con)?|$fin)(s)?$/))) { #allowing for tomma strengen

		print LOGG "DECIDED TO SPLIT $key from $wd in SPLIT_OR_NOT_AFTER_KEY\n" if $lg;

	    	#key-final -s (bruks)
	    	if ($key =~ /s$/) {
	    		$wd =~ s/$key/$key\+/;
	    		$wd =~ s/($con_no_s|e)s\+/$1~s\+/;
	    		print LOGG "RETURNING at 1\n";
	    		return $wd;
	    	}

	    	#key-final -e (vepse)
	    	if ($key =~ /e$/) {
	    		$wd =~ s/$key/$key\+/;
	    		$wd =~ s/($lc)e\+/$1~e\+/;
	    		print LOGG "RETURNING at 2\n";
	    		return $wd;
	    	}

	    	#afterkey-initial s- (vekt~s+klasse)
		if (($rest_aft_key =~ /^s($con)($con)/) & ($rest_aft_key !~ /^($spr_initial|$stj_initial|$str_initial|$spl_initial|$skr_initial|$spj_initial|$skv_initial|$skj_initial)/)) {
			$rest_aft_key =~ s/^s//;
		    	$wd =~ s/$key/$key~s\+/;
		    	$wd =~ s/~s\+s/~s\+/;
	    		print LOGG "RETURNING at 3\n";
	    		return $wd;
	    	}

	    	if ($wd =~ s/($key)e/$key~e\+/) {print LOGG "RETURNING at 4\n"; return $wd};

	    	$wd =~ s/$key/$key\+/;
		return $wd;
	}

	else {
		print LOGG "DECIDED NOT TO SPLIT $key from $wd in SPLIT_OR_NOT_AFTER_KEY\n" if $lg;
		return $wd;
	}

} #end sub split_or_not_after_key

##########################################################################################

#Analyses relationship between $wd and $key and determines whether to decompose before key or not.

sub split_or_not_before_key {
	my $wd = shift;
	my $key = shift;
	my $orig = $wd;

	print LOGG " Entered sub SPLIT_OR_NOT_BEFORE_KEY with word $wd and key $key\n" if $lg;

	@rest = split /$key/, $wd;
	$rest_bef_key = $rest[0];
	$rest_aft_key = $rest[1];
	$left_and_key = $rest_bef_key.$key;
	$key_and_right = $key.$rest_aft_key;

	print LOGG "If I decide to split, remnant \$rest_aft_key will be $rest_aft_key after key \n" if $lg;
	print LOGG "If I decide to split, remnant \$rest_bef_key will be $rest_bef_key before key \n" if $lg;
	print LOGG "If I decide to split, variable \$left_and_key will be $left_and_key before key \n" if $lg;
	print LOGG "If I decide to split, variable \$key_and_right will be $key_and_right after key \n" if $lg;

	foreach $k (keys (%not_cmp)) {
		return $wd if ($k =~ /^$left_and_key/);
		return $wd if ($k =~ /$key_and_right$/);	#struktur
	}
print LOGG "GOT TO 1!\n";
	return $wd if ($rest_aft_key =~ /^($suffix)/); 	#
print LOGG "GOT TO 2!\n";
	return $wd if ($rest_aft_key =~ /^([^es])$/); 	#
	return $wd if ($key_and_right =~ /^trete$/); 	#
print LOGG "GOT TO 3!\n";
	return $wd if ($rest_bef_key =~ /($prefix|^ans)$/) & ($rest_bef_key !~ /tekstil$/); 	# tilleggsstraff
print LOGG "GOT TO 4!\n";
	return $wd if ($rest_bef_key !~ /(($init)($vow)|e|($fin)($vow)|($vow)($db_con)(s)?|(($vow)($fin)(s)?)|(($vow)($con)(s)?))$/);
print LOGG "GOT TO 5!\n";
	if ($rest_aft_key) {
		return $wd if ($rest_aft_key !~ /^(~s|((s)?($init)($vow))|((s)?($con)($vow))|($vow))/); # don't if rest doesn't begin with legal initial cluster
	}

	if ($wd =~ /($lc){3}$key/) {		# don't if not preceded by three characters
		print LOGG "DECIDED TO SPLIT $key from $wd in SPLIT_OR_NOT_BEFORE_KEY\n" if $lg;
	    	if ($wd =~ s/e$key/~e\+$key/) {return $wd};
	    	if ($wd =~ s/($con_no_s)s$key/$1~s\+$key/) {return $wd};
	    	$wd =~ s/$key/\+$key/;
	    	print LOGG "before return \$wd = $wd\n" if $lg;
		return $wd;
	}

	elsif (($wd =~ /($lc){3}$key/) &		# don't if not preceded by three characters
	    not ($rest_aft_key)) {
		print LOGG "DECIDED TO SPLIT $key from $wd DUE TO EMPTY STRING in SPLIT_OR_NOT_BEFORE_KEY\n" if $lg;
	    	if ($wd =~ s/e$key/~e\+$key/) {return $wd};
	    	if ($wd =~ s/($con_no_s)s$key/$1~s\+$key/) {return $wd};
	    	$wd =~ s/$key/\+$key/;
		return $wd;
	}

	else {
		print LOGG "DECIDED NOT TO SPLIT $key from $wd in SPLIT_OR_NOT_BEFORE_KEY\n" if $lg;
		return $wd;
	}


} #end sub split_or_not_before_key
##########################################################################################

#Decide if s is part of following word or linking

sub check_s {
	my $wd = shift;
	my $cl = shift;
	my $orig = $wd;

	print LOGG "Entered sub check_s with word $wd and cluster $cl\n" if $lg;

	if ($wd =~ /^($uc)($lc)/) {
		$init_cap = 1;
		$wd = lc $wd;
	}

	print LOGG "$wd before return case\n" if $lg;
	return $wd;
} #end sub

##########################################################################################

1;
