#!usr/bin/perl

#Norwegian decomposition tool

use locale;
use POSIX 'locale_h';
setlocale(LC_CTYPE,$locale{Norwegian});

##########################################################################################
#Read command line arguments
$infile = $ARGV[0];
$outfile = $ARGV[1];

#open (IN, $infile) or die "Can't open $infile: $!\n";
#open OUT,">$outfile";

##########################################################################################
#Set user parameters
$lg = 0;					# log-writing on/off
open LOGG, ">logg.out" if $lg;

$show_infl = 0;					# show inflectional separators in output

##########################################################################################
#Open program code and lists required for decomposition
require '\.\\subs\\nordel_vars.pl';
require '\.\\subs\\nordel_subs.pl';
require '\.\\subs\\nordel_clusters.pl';
require '\.\\subs\\nordel_filelist.pl';

##########################################################################################
#Main loop

while (<STDIN>) {
	chomp;
	$line = $_;
	@words = split /\s/, $line;
	foreach $word (@words) {
#		print "$word\t";
		&init_vars;
		print LOGG "$wd\n" if $lg;
		next if ($word !~ /$lc/i);
		$word = &punctuation($word);
		$word = &case($word);
		$wn = &decompose($word);
		$wn = ucfirst $wn if $init_cap;
		print LOGG "Finally: $wn\n\n" if $lg;
		print LOGG "-" x 90, "\n" if $lg;
		print OUT "$wn\n";
		print "$wn\n";
	}
}

##########################################################################################
#Main decomposition subroutine

sub decompose {
	my $wd = shift;
	my $orig = $wd;

	$wd = &inflection($wd,$infl);
	($wd,$infl) = split "~", $wd;
	
	#Processing of entire word
	$wd = &hyphen($wd);
	$wd = &exception($wd);
	$wd = &pfx_sfx($wd);


	#Processing unit by unit

	#Lookup in rmcc-list
	@dec = split /\+/, $wd;
	$nmb = scalar @dec;
	for ($n = 0; $n <= $nmb; $n++) {
		if (($dec[$n] =~ /\w/) && ($dec[$n] !~ /($suffix)(\~(s|e))?$/)) {
			$dec[$n] = &common_rmcc($dec[$n]);
		}
	}
	$wd = join "\+", @dec;


	#Lookup in lmcc-lists
	@dec = split /\+/, $wd;
	$nmb = scalar @dec;
	for ($n = 0; $n <= $nmb; $n++) {
		if (($dec[$n] =~ /\w/) && ($dec[$n] !~ /($suffix)(\~(s|e))?$/)) {
			$dec[$n] = &common_lmcc($dec[$n]);
		}
	}
	$wd = join "\+", @dec;
	

	#Cluster decomposition
	@dec = split /\+/, $wd;
	$nmb = scalar @dec;
	for ($n = 0; $n <= $nmb; $n++) {
		if (($dec[$n] =~ /\w/) && ($dec[$n] !~ /~(s|e)$/)) {
			$dec[$n] = &cluster($dec[$n]) if $dec[$n];
		}
	}
	$wd = join "\+", @dec;
	
	if ($show_infl) {
		$wd = $wd."~".$infl if $infl;
	}
	else {
		$wd = $wd.$infl if $infl;
	}

	return $wd;
} #end sub
##########################################################################################
