#!usr/bin/perl

#Subroutines required for Norwegian decomposition tool

##########################################################################################
#Strip punctuation

sub punctuation {
	my $wd = shift;
	my $orig = $wd;

	$wd =~ s/[\.\,\:\;\!\?\"\'\(\)\{\}\[\]]//g;

	print LOGG "$wd before return punctuation\n" if $lg;
	return $wd;
} #end sub



##########################################################################################
#Strip punctuation

sub case {
	my $wd = shift;
	my $orig = $wd;

	if ($wd =~ /^($uc)($lc)/) {
		$init_cap = 1;
		$wd = lc $wd;
	}

	print LOGG "$wd before return case\n" if $lg;
	return $wd;
} #end sub


##########################################################################################
#Strip inflections

sub inflection {
	my $wd = shift;
	my $orig = $wd;

	return $wd if $wd =~ /($db_con)$/;
	$wd =~ s/(...)($inflection)$/$1~$2/;

	print LOGG "$wd before return inflection\n" if $lg;
	return $wd;
} #end sub

##########################################################################################
#Handle hyphenated compounds

sub hyphen {
	my $wd = shift;
	my $orig = $wd;
	
	return $wd if $wd !~ /\-/;
	
	$wd =~ s/\-/\+\-\+/g;

	print LOGG "$wd before return hyphen\n" if $lg;
	return $wd;
} #end sub hyphen

##########################################################################################
#Handle exceptions

sub exception {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "$wd before return exception\n" if $lg;
	return $wd;
} #end sub exception

##########################################################################################
#Handle compounds with common derivational suffixes

sub pfx_sfx {
	my $wd = shift;
	my $orig = $wd;
	
	$wd =~ s/(...)($suffix)s(...)/$1$2~s\+$3/g;		# kristendom~s+kunnskap
	$wd =~ s/(...)($suffix_u_s)(...)/$1$2\+$3/g;		# walisisk+liknende
	$wd =~ s/(...)s($prefix)(...)/$1~s\+$2$3/g;		# lnn~s+oppgjr
	$wd =~ s/(...)($prefix)(...)/$1\+$2$3/g;		# fregatt+oppdrag
	
	$wd =~ s/\+\+/\+/g;

	print LOGG "$wd before return pfx_sfx\n" if $lg;
	return $wd;
} #end sub pfx_sfx

##########################################################################################
#Handle compounds with common rightmost component

sub common_rmcc {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered common_rmcc with $wd\n" if $lg;
	
	#Testing for rightmost component
	foreach $key (keys (%cm_rmcc)) {
		$wd =~ s/(...)$key/$1\+$key/;		# hoved+vei
		$wd =~ s/(...)(e)\+$key$/$1~e\+$key/;		# land+e+vei
		$wd =~ s/(...)(s)\+$key$/$1~s\+$key/;		# ferdsel+s+vei
	}

	print LOGG "$wd before return common_rmcc\n" if $lg;
	return $wd;
} #end sub rmcc

##########################################################################################

#Handle compounds with common leftmost component
#on the basis of entry lists ending with 1 2 3 or 4 consonants

sub common_lmcc {
	my $wd = shift;
	my $orig = $wd;

	print LOGG "Entered common_lmcc with $wd\n" if $lg;

	print LOGG "$wd before replacement of list item from con_4\n" if $lg;
	#Testing for leftmost component
	foreach $key (keys (%con_4)) {
		if ($wd =~ s/($key)s($scls)/$1\+s$2/)	{return $wd};		# angst+skrik
		if ($wd =~ s/($key)s(..)/$1~s\+$2/)	{return $wd};		# 
		if ($wd =~ s/($key)e(..)/$1~e\+$2/)	{return $wd};		# 
		if ($wd =~ s/($key)(..)/$1\+$2/)	{return $wd};		# 
	}

	print LOGG "$wd before replacement of list item from con_3\n" if $lg;
	foreach $key (keys (%con_3)) {
		if ($wd =~ s/($key)s($scls)/$1\+s$2/)	{return $wd};		# 
		if ($wd =~ s/($key)s(..)/$1~s\+$2/)	{return $wd};		# 
		if ($wd =~ s/($key)e(..)/$1~e\+$2/)	{return $wd};		# 
		if ($wd =~ s/($key)(..)/$1\+$2/)	{return $wd};		# 
	}

	print LOGG "$wd before replacement of list item from con_2\n" if $lg;
	foreach $key (keys (%con_2)) {
		if ($wd =~ s/($key)s($scls)/$1\+s$2/)	{return $wd};		# kost+skole
		if ($wd =~ s/($key)s(..)/$1~s\+$2/)	{return $wd};		# 
		if ($wd =~ s/($key)e(..)/$1~e\+$2/)	{return $wd};		# 
		if ($wd =~ s/($key)(..)/$1\+$2/)	{return $wd};		# 
	}

	print LOGG "$wd before replacement of list item from con_1\n" if $lg;
	foreach $key (keys (%con_1)) {
		if ($wd =~ s/($key)s($scls)/$1\+s$2/)	{return $wd};		# 
		if ($wd =~ s/($key)s(..)/$1~s\+$2/)	{return $wd};		# liv~s+glad
		if ($wd =~ s/($key)e(..)/$1~e\+$2/)	{return $wd};		# hp~e+full
		if ($wd =~ s/($key)(..)/$1\+$2/)	{return $wd};		# hoved+vei
	}

	print LOGG "$wd before return consonant_lists\n" if $lg;
	return $wd;
} #end sub rmcc

##########################################################################################


1;
