#! perl

## Filename:	trcParser_no.pl
## Author:		Sofia Strmbergsson
## Date:		000228
##
## Description:
##
## Including files:	no_parsing_parameters.pl
##		no_lexicon.pl
##
## Last updated:	000405:	Added $NOR_ALLOWED_DUPLICATION
##			Added %NOR_COMB_CONDITION
##			Added counter to output:	$nofLines = 0;
##						$interval = 500;
##						$newInterval = $interval;
##			Changed 		if($morf !~ /\+/ ||$morf !~ /$NOR_DERIV/) { (gemination)
##			To:		if($morf !~ /\+/ && $morf !~ /$NOR_DERIV/) {
##			Changed whole routine for finding rare and illegal stress patterns
##			Changed lines 491 and 499 and 248
##
## Last updated	000406:	Removed line 	$syll =~ s/^_//;
##			Replaced lines	$syll =~ s/^2'//g;
##					$syll =~ s/^'2?//g;
##			with:		my($tmp) = $NOR_NOT_SOUND;
##					$tmp =~ s/\|//g;
##					$syll =~ s/^[$tmp]+//g;
##			Replaced line	if($morf !~ /($NOR_DERIV)/ && $morf !~ /\+/) {
##			with		if($morf !~ /($NOR_DERIV|$NOR_COMPOUND_MARKER)/) {
##			Added procedure for printing statistics to STDOUT
## Last updated	000407:	Removed the possibility of agreeing with MOP-errors in words that are not marked as compounds
##			Changed the MOP-violation procedure
## Last updated	000408 & 000409:	Changed TRC-correspondence procedure
## Last updated	000414	Added different reading and writing modes, so that the parser might parse its own (corrected) output
## Last updated	000418	Removed the following lines:
##					$trc =~ s/&;/&/g;		# remove all ;-chars following the tie bar, i e &
##					$trc =~ s/%;(.)/%$1/g;		# remove all ;-chars following the %-char
##			Removed the PARSED/UNPARSED-distinction

	






# include lexicon file and file with search parameters
require "no_lexicon.pl";
require "no_parsing_parameters.pl";


###################################################
# CALL WITH ARGUMENT
###################################################
# if the call to the parser had an argument
if(@ARGV[0]) {
	my $inputString = @ARGV[0];
	@posts = split(/;/, $inputString);    	# split each line at ;-chars and collect in the trcs-array
	$morf = $posts[2];          		# initialize morf to the morfological decomp
	$pos = $posts[3];           		# initialize pos to the pos
	$lang = $posts[6];		# initialize lang to the 6th post
	$garbage = $posts[7];		# initialize garbage to post 8 (0 if not garbage, otherwise 1)
	$expan = $posts[10];		# initialize expan to the 11th post (expansions of acronyms)
	@trcs = ($posts[11], $posts[15], $posts[19], $posts[23]);	# initialize trcs-array to posts containing transcriptions
	$trcset = $posts[29];		# initialize trcset to the name of the set of wich the transcription is a member
	$lineIndex = $posts[40];		# initialize lineIndex ot the 41th pos
	chop($lineIndex);

	if($garbage == $VALUE_FOR_GARBAGE && $trcset =~ /$SELECTION_OF_TRANSCRIPTIONS/) { 
		foreach $trc (@trcs) {
			$errDesc = "";
			if($trc ne "") {
				$nofTrcs++;
				$errDesc .= &symbolParse($trc, $morf);	# parse the symbol structure of trc1
				$errDesc .= &syllStressParse($trc, $morf);	# parse the syllable and stress structure
				if($errDesc ne "") {
					my $newErrDesc = "";
					$trc =~ s/;//g;		# remove all ;-chars from the transcription string
					my @errDesx1 = split(/ ::/, $errDesc);
					my @errDesx2 = sort(@errDesx1);
					foreach $errDesc (@errDesx2) {$newErrDesc .= "\n\t$errDesc";}
					my $totString = "$trc;$lineIndex;$trcIndex;$pos;$morf;$lang;$expan;$newErrDesc\n\n";
					print STDOUT "\n\n$totString\n\n";
				}
			}
		}
	}
}



###################################################
# CALL WITHOUT ARGUMENT = READ FROM FILE
###################################################
# else, if the call to the parser had NO argument...
else {
	# erase what's allready in OUTFILE
	open(OUT, ">$OUTFILE");
	print OUT "";
	close(OUT);

	# set counters to initial value 0
	my $nofLines = 0;
	my $nofTrcs = 0;
	my $nofErrTrcs = 0;
	my $nofGarbage = 0;

	# decide how often counting message should be printed to output
	my $interval = 250;
	my $newInterval = $interval;
	my $correctStart = 0;
	
	open(TRANSCRIPTIONS, $INFILE);		# open the input file
	while (<TRANSCRIPTIONS>) {
		my $lexEntry = $_;
		if($lexEntry =~ /CORRECT\sENTRIES/) {$correctStart = 1;}
		# if the current line contains valuable information...
		elsif($lexEntry =~ /^[^\s\t]/ && $correctStart == 0) {
			@posts = split(/;/, $lexEntry);    	# split each line at ;-chars and collect in the trcs-array
			$morf = $posts[2];          		# initialize morf to the morfological decomp
			$pos = $posts[3];           		# initialize pos to the pos
			$lang = $posts[6];		# initialize lang to the 6th post
			$garbage = $posts[7];		# initialize garbage to post 8 (0 if not garbage, otherwise 1)
			$expan = $posts[10];		# initialize expan to the 11th post (expansions of acronyms)
			@trcs = ($posts[11], $posts[15], $posts[19], $posts[23]);	# initialize trcs-array to posts containing transcriptions
			$trcset = $posts[35];		# initialize trcset to the name of the set of wich the transcription is a member
			$lineIndex = $posts[40];		# initialize lineIndex ot the 41th pos
			chop $lineIndex;

			$trcIndex = 1;	
			if($garbage == 1) {$nofGarbage++;}
		
			if($garbage == $VALUE_FOR_GARBAGE && $trcset =~ /$SELECTION_OF_TRANSCRIPTIONS/) { 
				foreach $trc (@trcs) {
					$errDesc = "";
					if($trc ne "") {
						$nofTrcs++;
						$errDesc .= &symbolParse($trc, $morf);	# parse the symbol structure of trc1
						$errDesc .= &syllStressParse($trc, $morf);	# parse the syllable and stress structure
						if($errDesc ne "") {
							$nofErrTrcs++;
							my $newErrDesc = "";
							$trc =~ s/;//g;		# remove all ;-chars from the transcription string
							my @errDesx1 = split(/ ::/, $errDesc);
							my @errDesx2 = sort(@errDesx1);
							foreach $errDesc (@errDesx2) {$newErrDesc .= "\n\t$errDesc";}
							if($NICE_OUTPUT == 1) {
								my $totString = "$trc;$lineIndex;$trcIndex;$pos;$morf;$lang;$expan;$newErrDesc\n\n";
								$errOut{$totString} = $newErrDesc;
							}
							else {
								chop $lexEntry;
								my $totString = "$lexEntry"."$newErrDesc\n\n";
								$errOut{$totString} = $newErrDesc;
							}
						}
						# if the lexical entry contains no found errors...
						elsif($lexEntry ne "") {push(@correctOut, $lexEntry);}
					}
					$trcIndex++;
				}	
			}
		}
		# if the lexical entry is already parsed and no errors have been found
		elsif($correctStart == 1 && $lexEntry ne "") {push(@correctOut, $lexEntry);}
		# counter information passed onto the user
		if($nofLines == $newInterval) {
			print STDOUT "$nofLines lines parsed\n";
			$newInterval += $interval;
		}
		$nofLines++;
	} # end of while (<TRANSCRIPTIONS>) 
	close(TRANSCRIPTIONS);
	
	#############################################################
	# PRINTING STATISTICS TO STDOUT
	#############################################################
	print STDOUT "Total number of lexical entries in $INFILE = $nofLines\n";
	print STDOUT "Total number of transcriptions in $INFILE = $nofTrcs\n";
	print STDOUT "Total number of erroneous transcriptions found= $nofErrTrcs\n";
	print STDOUT "Number of transcriptions for which the garbage value equals 1 = $nofGarbage\n";
	print STDOUT "Output printed to file $OUTFILE\n";
	
	############################################################
	# SORTING AND PRINTING OUTPUT MESSAGES
	############################################################
	# messages containing just warnings are separated from those containing errors
	foreach $errString (keys %errOut) {
		if($errString !~ /e\s=\s/) {push(@warnings, $errString);}
		else {push(@mixed, $errString);}
	}
	
	# the keys in %errOut are sorted by values
	@mixedKeys = sort {$errOut{$a} cmp $errOut{$b}} @mixed;
	@warningKeys = sort {$errOut{$a} cmp $errOut{$b}} @warnings;
	
	open(OUT, ">>$OUTFILE");
	# output is printed to file
	foreach $mix (@mixedKeys) {print OUT $mix;}
	foreach $warning (@warningKeys) {print OUT $warning;}
#	print OUT "\n\nCORRECT ENTRIES:";
#	@sortedCorrect = sort @correctOut;
#	foreach $correct (@sortedCorrect) {print OUT "$correct";}
	close(OUT);
}	



###########################################################
# PARSING SUBROUTINES
###########################################################
###########################################################
# symbolParse
# DESCR:	the trc string is divided in symbol and if errors are 
#	found, these are collected in an error description 
#	string. In general terms, the routine looks for plain
#	symbol errors, using no linguistic knowledge.
# INPUT:	trc string, morphological decomposition string
# OUTPUT: 	error description string
###########################################################
sub symbolParse {
	# take care of incoming variables
	my $trc = $_[0];
	my $morf = $_[1];
	my $errDesc = "";

	# remove all allowed duplications from the transcription string
	my $tmp = $trc;
	$tmp =~ s/($NOR_ALLOWED_DUPLICATION)//g;

	# check that no character is duplicated (it should be by mistake?)
	if($tmp =~ /(.);?\1/) {
		if($W_UNWANTED_DUPLICATION == 1) {$errDesc .= 'w = probably unwanted duplication of '.$1.' ::';}
	}

	# if trc ends with a syllable delimiter
	if($trc =~ /($NOR_SYLLABLE_DELIMITER)$/) {
		if($E_PERIOD_LAST == 1) {$errDesc .= 'e = illegal position for syllable delimiter ::';}
	}

	# check that the order of the characters in trc is allowed in the specific language
	# and that expected correspondance to some substring in morf is found in trc
	$errDesc .= &checkChars($trc, $morf);
	
	# check that there are as many '_':s in the transcription string as in the morphological decomposition string
	$_ = $trc; my $nofInTrc = tr/\_/\_/;
	$_ = $morf; my $nofInMorf = tr/\_/\_/;	
	if($nofInTrc != $nofInMorf){
		if($E_NOF_UNDERSCORES == 1) {$errDesc .= "e = the number of '_':s in transcription is not correct ::";}
	}

	# extract symbols in the transcription string by inserting ;-chars before every symbol
	$trc =~ s/($NOR_SYMBOL)/;$1/g;
	# insert a ;-char string-initially (for those strings that don't already start with a ;-char)
	$trc =~ s/^([^;])/;$1/;		

	my $inside = $trc;		# initialize inside to the trc-string
	$inside =~ s/[^%]*%"([^"]+)".*$/$1/;	# remove everything but the chars within the ""-chars
	$inside =~ s/;//g;		# remove the ;-chars from the string
	$trc =~ s/%"([^"]+)"/%"$inside"/;	# re-insert the inside-string in the trc-string
	

	# split the transcription by ;-chars and collect the symbols in an array
	my @symbols = split(/\;/, $trc);
	# parse each symbol
	foreach $symbol (@symbols) {
		# check that the symbol is legal for the current language
		if($symbol ne "") {
			if($symbol !~ /^($NOR_SYMBOL)$/) {
				if($E_ILLEGAL_NOR_SYMBOL == 1) {
					$errDesc .= "e =  illegal Norwegian symbol ($symbol) ::";
				}
			}
			if($symbol !~ /^.$/) {$errDesc .= &legalSymbol($symbol, $trc);}
		}
	}
	$errDesc;
}# end of symbolParse




###########################################################
# legalSymbol
# DESCR:	divides the symbol into head, tail and diacritics. At the
#	moment nothing is done with the head or the tail, but
#	the routine finds ill-formed diacritics.
# INPUT:	a (potential) symbol, the trc string
# OUTPUT: 	error description string
###########################################################
sub legalSymbol {
	my $symbol = $_[0];
	my $trc = $_[1];
	my $errDesc = "";

	my $head = $symbol;		# initialize head to the whole symbol string
	my $tail = "";		# initialize tail to empty string
	my $diacs = "";		# initialize diacs to empty string

	# if the head-string contains any %-char
	if($head =~ /%/) {
		$diacs = "\%".$';	# initialize diacs to the % + the string following it
		$head = $`;		# re-initialize head to the string without diacritics
	}

	# if the head-string contains any TAIL-chars
	if($head =~ /$TAIL_CHARS/) {
		$_ = $head;
		/^($HEAD_CHARS|&)+/;
		# initialize tail to the first non-HEAD (exept &!) in the
		# head-string and all characters thereafter
		$tail = $';
		# initialize the head-string to the first HEAD-char
		$head = $&;
	}

	# if the diacs-string is not empty
	if($diacs ne "") {
		# if the diacs-structure is not legal
		if($diacs !~ /^$DIACS_STRUCTURE$/) {
		   	if($E_ILL_FORMED_DIACRITIC == 1) {$errDesc = "e = contains illegal diacritic $diacs ::";}
		}
	}# end of if($diacs ne "")
	$errDesc;
}# end of legalSymbol


###########################################################
# syllStressParse
# DESCR:	checks the stress pattern of the transcription and if the
#	syllable structure is legal
# INPUT:	the trc string, the morphological decomposition string
# OUTPUT: 	error description string
###########################################################
sub syllStressParse {
	my $trc = $_[0];
	my $morf = $_[1];
	my $errDesc = "";
	my $syllIndex = 0;         # set syllable index to 0, i e the first syllable of the word
	
	# if the word is marked with secondary stress	
	if($trc =~ /$NOR_SEC_STRESS_MARK/) {
		# if the word is not a compound word or a derived word, 
		# i e the transcription should not be transcribed as a compound
		if($morf !~ /($NOR_DERIV|$NOR_COMPOUND_MARKER)/) {
			if($W_SIMPLEX_WITH_SEC_STRESS == 1) {$errDesc .= 'w = possible non-compound with \'2 ::';}
		}
	}
	
	my @sylls = split(/$NOR_SYLLABLE_DELIMITER/,$trc);	# split the transcription at ".":s and "_":s
	
	my $lastSyll = "";				# initialize lastSyll to empty string

	# each syllable is checked so that it contains a vowel, otherwise a warning is printed
	foreach $syll (@sylls) {
		my $countVows = &countVowels($syll);
		# if the syllable doesn't contain any vowel or the sequence %)
		if($countVows == 0 && $syll !~ /$NOR_SYLLABIFIED_CONS/) {
			if($E_NO_NUCLEUS == 1) {$errDesc .= 'e = syllable contains no nucleus ('.$syll.') ::';}
			$lastSyll = "";
		}
		# if the syllable contains more than one nucleus
		elsif($countVows > 1 || ($countVows == 1 && $syll =~ /$NOR_SYLLABIFIED_CONS/)) {
		       	if($E_TOO_MANY_NUCLEI == 1) {$errDesc .= 'e = syllable contains more than one nucleus ('.$syll.') ::';}
			$lastSyll = "";
		}
		else {
			$errDesc .= &syllCheck($trc, $syll, $syllIndex);
			if($lastSyll ne "") {$errDesc .= &mopCheck($morf, $lastSyll, $syll);}
			$lastSyll = $syll;
		}
		$syllIndex++;
	}	
	
	# initialize stringOk to 1
	my $stringOk = 1;
	# if the transcription contains some word delimiter
	if($trc =~ /$NOR_WORD_DELIMITER/) {
		# split the phrase by word delimiter
		@words = split(/$NOR_WORD_DELIMITER/, $trc);
		# set the word delimiter counter to 0
		my $nofPhraseAccents = 0;
		foreach $word (@words) {
			if($word =~ /^($NOR_PHRASE_ACCENT)$/) {
				$nofPhraseAccents++;}
			elsif($word !~ /^($NOR_TONEME1|$NOR_TONEME2)$/) {$stringOk = 0;}
		}
		if($nofPhraseAccents != 1) {$stringOk = 0;}
	}
	# if the transcription doesn't match either toneme1 or toneme2
	elsif($trc !~ /^($NOR_TONEME1|$NOR_TONEME2)$/) {
		while(($rarePattern, $conditions) = each %NOR_COMB_CONDITION) {
			# if the transcription matches a rare pattern
			if($trc =~ /^($rarePattern)$/) {
				($operator, $morfCond, $expanCond, $posCond) = split(/\;/, $conditions);
				# if the logical operator is AND...
				if($operator eq "AND") {
					if($morfCond =~ /($NOR_COMPOUND_MARKER)/) {
						$morfCond .= '|'.$NOR_DERIV;
					}
					# ... and all conditions are NOT fullfilled
					if(!($morf =~ /($morfCond)/ && $expan =~ /($expanCond)/ && $pos =~ /($posCond)/)) {
						$stringOk = 0;
					}
				}
				# if the logical operator is OR...
				elsif($operator eq "OR") {
					if($morfCond =~ /($NOR_COMPOUND_MARKER)/) {
						$morfCond .= '|'.$NOR_DERIV;
					}
					#... and none of the conditions is fullfilled
					if($morf !~ /($morfCond)/ && $expan !~ /($expanCond)/ && $pos !~ /($posCond)/) {
						$stringOk = 0;
					}
				}
			}
		}
	}
	if($stringOk == 0 && $E_ILLEGAL_STRESS_PATTERN == 1) {
		$errDesc .= 'e = contains illegal stress pattern ::';
	}
	$errDesc;
}# end of sub syllStressParse




###########################################################
# checkChars
# DESCR:	checks for unwanted sequences of characters, i.e.
#	if the transcription conventions are violated.
# INPUT:	the trc string, the morphological decomposition string
# OUTPUT: 	error description string
###########################################################
sub checkChars {
	my $trc = $_[0];
	my $morf = $_[1];
	my $errDesc = "";

	$trc =~ s/;//g;             # remove all ;-chars
	
	while(($pre,$post) = each %NOR_X_CANNOT_FOLLOW_Y) {
		if($trc =~ /$pre$post/) {
			if($E_UNWANTED_SEQUENCE == 1) {$errDesc .= 'e = unwanted sequence of '.$3.' and '.$5.''.$7.' ::';}
		}
	}
	while(($pre,$post) = each %NOR_X_SHOULD_NOT_FOLLOW_Y) {
		if($trc =~ /$pre$post/) {
			if($W_UNWANTED_SEQUENCE == 1) {$errDesc .= 'w = probably unwanted sequence of '.$3.' and '.$5.''.$7.' ::';}
		}
	}	
	while(($ortCorr,$trcCorr) = each %TRC_X_CORRESPONDS_TO_ORTO_Y) {
		if($morf =~ /$ortCorr/) {
			$ortCorr = $1.$3;
			if($trc !~ /$trcCorr/) {
				# remove everything before (X in /.*(X)(Y)(Z).*/
				$trcCorr =~ s/^[^\(]*//;
				# remove everything after Z) in /X)(Y)(Z).*/
				$trcCorr =~ s/[^\(\)]*$//;
				# remove everything between (X) and (Z) in /(X)(Y)(Z)/
				$trcCorr =~ s/^([^\)]*\))[^\(]*\([^\)]*\)[^\(]*(\(.*)$/$1$2/;
				# remove all \-chars
				$trcCorr =~ s/\\([^0-9])/$1/g;
				if($W_ORTOGRAPHIC_CORRESPONDENCE == 1) {
					$errDesc .= 'w = correspondence /'.$trcCorr.'/ to <'.$ortCorr.'> not found ::';
				}
			}
		}
	}
	while(($trcCorr,$langCorr) = each %NOR_TRC_IFF_LANG) {
		if($trc =~ /$trcCorr/ && $lang !~ /$langCorr/) {
			if($W_LANG_CORRESPONDENCE == 1) {
				$errDesc .= 'w = /'.$trcCorr.'/ allowed only in words with language code '.$langCorr.' ::';
			}
		}
	}
	if($W_NOT_GEMINATE_CONS == 1) {
		# if the word is not a compound or a derived word
		if($morf !~ /\+/ && $morf !~ /$NOR_DERIV/) {
			# if the same consonant occurs on both sides of a syllable boundary
			if($trc =~ /($NOR_CONSONANT)($NOR_NOT_SOUND)+($NOR_CONSONANT)/ && $1 eq $3) {
				my $cons = $1;
				$errDesc .= 'w = geminate consonant '.$cons.' over syllable boundary ::';
			}
		}
	}
	$errDesc;
}




###########################################################
# syllCheck
# DESCR:	checks for illegal or rare onsets and codas
# INPUT:	the trc string, a syllable, the index of the syllable
# OUTPUT: 	error description string
###########################################################
sub syllCheck {
	my $trc = $_[0];
	my $syll = $_[1];
	my $syllIndex = $_[2];
	my $errDesc = "";
	
	my $onset; my $coda;

	my $tmp = $NOR_NOT_SOUND;
	$tmp =~ s/\|//g;
	$syll =~ s/^[$tmp]+//g;

	($onset, $coda) = split(/$NOR_VOWEL|$NOR_SYLLABIFIED_CONS/, $syll);

	if($onset ne "") {
		if($onset =~ /$NOR_RARE_ONSET/) {
			if($W_RARE_NOR_ONSET == 1) {$errDesc .= "w = rare Norwegian onset ($onset) ::";}
		}
		elsif($syllIndex == 0 && $onset =~ /$NOR_NOT_INITIAL_ONSET/) {
			if($E_ONSET_NOT_INITIALLY == 1) {$errDesc .= "e = onset is not allowed word-initially ($onset) ::";}
		}
		elsif($onset !~ /$NOR_LEGAL_ONSET/) {
			if($E_NOT_NOR_ONSET == 1) {$errDesc .= "e = not a Norwegian onset ($onset) ::";}
		}
	}
	if($coda ne "") {
		if($coda =~ /$NOR_RARE_CODA/) {
			if($W_RARE_NOR_CODA == 1) {$errDesc .= "w = rare Norwegian coda ($coda) ::";}
		}
		elsif($coda !~ /$NOR_LEGAL_CODA/) {
			if($E_NOT_NOR_CODA == 1) {$errDesc .= "e = not a Norwegian coda ($coda) ::";}
		}
	}
	$errDesc;
}



###########################################################
# countVowels
# DESCR:	counts the number of vowel in the given syllable
# INPUT:	a syllalbe
# OUTPUT: 	the number of vowels in the syllable
###########################################################
sub countVowels {
	my $inString = $_[0];
	my $countVows = 0;
	
	# remove any diacritics, since there might be vowels in the diacs-string
	# and these should not be counted
	$inString =~ s/([^%]+)$DIACS_STRUCTURE(.*)$/$1$3/g;

	# if the string contains diphgongs
	while($inString =~ s/$NOR_DIPHTHONG//) {$countVows++;}

	# count the number of legal vowels in the syllable
	while($inString =~ s/$NOR_VOWEL//) {$countVows++;}
	$countVows;                 # return the number of legal vowels found
}




###########################################################
# mopCheck:
# DESCR:	checks two syllables at a time and prints an error 
#	message if a MOP-violation is found. Exceptions are
#	compound and derived words and abbreviations of
#	the type "AMF".
# INPUT:	the morphological decomposition string, two successive
#	syllables
# OUTPUT: 	an error description string
###########################################################
sub mopCheck {
	my $morf = $_[0];
	my $syll1 = $_[1];
	my $syll2 = $_[2];
	my $errDesc = "";
	
	# if the first syllable ends with at least one consonant
	if($syll1 =~ /($NOR_VOWEL)($NOR_CONSONANT)+$/) {
		my $comb = $syll1.$syll2;  		# concatenate syll1 and syll2
		
		# extract the onset of $syll2:
		$syll2Onset = $syll2; 
		$syll2Onset =~ s/$NOR_NOT_SOUND//g;  	# remove all 2:s and ':s
		$syll2Onset =~ s/^(($NOR_CONSONANT)*).*/$1/;

		# remove onset+vowel from syll1 and rhyme from syll2
		$comb =~ s/$NOR_NOT_SOUND//g;  # remove all 2:s and ':s
		$comb =~ s/^(.*?)($NOR_VOWEL)(($NOR_CONSONANT)*).*$/$3/;
		
		# no MOP-error has yet been found
		my $errorFound = 0;

		# while no MOP-error has been found and while $comb is not empty or equal to $syll2Onset...
		while($errorFound == 0 && $comb ne $syll2Onset && $comb ne "") {
			$^W = 0; 					# (do not print a warning)
			# if $comb forms a legal onset, i.e. a MOP-violation has been found
			if($comb =~ /$NOR_LEGAL_ONSET/) {
				# if the word is not marked as a compound and is not an abbreviation
				if($morf !~ /($NOR_COMPOUND_MARKER)/ && $morf !~ /[A-Z]{2,}/ && $expan eq "") {
					if($E_MOP_VIOLATION == 1) {
						$errDesc = 'e = MOP-violation, syllables '.$syll1.' and '.$syll2.' ::';
					}
				}
				# if the word is marked as a compound or is an abbreviation (and contains a MOP-violation, could be at the wrong place...)
				elsif($W_MOP_VIOLATION == 1) {
					$errDesc = 'w = possible MOP-violation, syllables '.$syll1.' and '.$syll2.' ::';
				}
				$errorFound = 1;
			}
			# remove initial consonant from $comb
			else {$comb =~ s/^$NOR_CONSONANT(.*)/$1/;}
		}
	}
	$errDesc;
}