#!perl 
use locale;
use POSIX 'locale_h';
setlocale(LC_CTYPE,$locale{Swedish});

# VERSION 6:
# Updates version from v4.
# Accepts multiword tokens '"' and '_'. (just_doit2 has an ugly fix to remove syllable marker)
# Stem processes for trk now returns the nr of syllables in the stem for to be used by retroflexing and syllabifying processes
# Possibility to see the whole post in the gui from menu 'settings'.
# This version should be used from 00.08.29 by Charlotta and Cattis
# END VERSION 6

# VERSION 6b:
# Updates version from v6.
# Changed some printing variables to allow a non_grafical shell
# and routines for printing lexicon posts for the inflected forms
# ($printarild)
# 
# END VERSION 6b
#
# VERSION 6c:
# Changed the functionality of sub vowel_shortening to operate on
# the last syll in the stem and onwards.
#
# END VERSION 6c
#
#
# VERSION 7:
# Lang specific file changed to handle n->nK when meeting k or g in the split between stems and endings.
# New rulefile used for validating s0 12 sept
#
#
# VERSION 8:
#
#
#
#
#
#
#
#
#

###########################
#                         #
#                         #
#          USE            #
#                         #
#                         #
###########################

use Cwd;
use Tk;
use Tk::Dialog;
use Tk::DialogBox;
use Tk::MultiMenu;
use BrowseEntryLW;
use vars qw/$TOP/;


###########################
#                         #
#       CONSTANTS         #
#          AND            #
#         INITS           #
#                         #
###########################

### ---NORSKA--- ###
require 'VIPSnor_v4.pl'; 				# 
require 'VIPS_v8.pl'; 					# 
require 'guinflect_help.pl';
$rulefile = "VIPSnor_v4.rule"; 			# 





### ---SVENSKA--- ###
#require 'VIPSswe_v8.pl'; 				# was VIPSswe_v7.pl
#require 'VIPS_v8.pl'; 					# was VIPS_v7.pl
#require 'guinflect_help.pl';
#$rulefile = "LW27aug_VIPSswe_v4.rule";
#$rulefile = "CK_VIPSswe_v4.rule";
#$rulefile = "VIPSswe_v8.rule"; 			# was VIPSswe_v7b.rule 


$locktext = 1;
$printlog = 0;
$printstems = 0;
$winlog = 1;
$printarild = 0;





$job = "no_job";
$currlang = "sw";
%raw_rules = &MkRuleHash($rulefile);
%rules = &PreprocessRule(%raw_rules);
#print LOG "Opening a lexiconfile \'$ARGV[1]\' to process\n\n" if $printlog;
$lexiconfile = "No lexicon file opened";
$filetypes = [
	      ['lexicon files', ['.lex']],
	      ['All files', ['*.*']]
	      ];
@lexlista = ();
@posttext = ();
@searchtext = ();
@paradigms = (["s0 egennavn", "s0"], ["s1 masc_er", "s1"], ["s2 masc_e", "s2"], ["s3 masc_uten_endelse", "s3"], ["s4 masc_uten_flertall", "s4"], ["s5 masc_uregelrett", "s5"], ["s6 fem_er", "s6"], ["s7 fem_e", "s7"], ["s8 fem_uten_endelse", "s8"], ["s9 fem_r", "s9"], ["s10 fem_uten_flertall", "s10"], ["s11 neu_er", "s11"], ["s12 neu_e", "s12"], ["s13 neu_uten_endelse", "s13"], ["s14 neu_r", "s14"], ["s15 neu_um", "s15"], ["s16 neu_uten_flertall", "s16"], ["s17 neu_uten_entall", "s17"], ["s20 masc_ubyelig", "s20"], ["s21 fem_ubyelig", "s21"], ["s22 neu_ubyelig", "s22"]);


#svensk: @paradigms = (["s1   -or", "s1"], ["s1-tre   -or", "s1-tre"], ["s2   -ar", "s2"], ["s2-tre   -ar", "s2-tre"], ["s3   -er,utrum", "s3"], ["s3-ett   -er,utrum", "s3-ett"], ["s3-tre   -er,utrum", "s3-tre"], ["s4   -er,neutrum", "s4"], ["s4-tre   -er,neutrum", "s4-tre"], ["s5   -r", "s5"], ["s5-tre   -r", "s5-tre"], ["s6   -n", "s6"], ["s7   -0,neutrum", "s7"],["s10   -0,utrum", "s10"], ["s8   -noplur,neutrum", "s8"], ["s9   -noplur,utrum", "s9"], ["s11   -irreg", "s11"],["s110   -irreg,neutrum", "s110"], ["s111   -irreg,utrum", "s111"], ["s20   -nosing", "s20"], ["v1   pres-r; pret-de", "v1"], ["v2   pres-er; pret-de", "v2"], ["v3   pres-er; pret-te", "v3"], ["v4   pres-r; pret-dde", "v4"], ["v5   -irreg", "v5"], ["a1   -vacker", "a1"], ["a1-tre   -", "a1-tre"], ["a2   -blt", "a2"], ["a2-tre   -", "a2-tre"], ["a24   -fri", "a24"], ["a24-tre   ", "a24-tre"], ["a3   -beige", "a3"], ["a3-tre   ", "a3-tre"], ["a4   -arbetsfri", "a4"], ["a6   -lng", "a6"], ["a6-tre-   ", "a6-tre"]);
#[paradigm label, paradigm value]

@defComm = ('status=OK', 'status=NOT_OK_regelfeil', 'status=NOT_OK_nonsenseword', 'status=NOT_OK_review');

#@poschoices = ("to be converted", "noun", "verb", "adj");
@poschoices = qw/noun verb adj adverb preposition pronoun conjunction noun noun noun noun noun noun noun noun noun noun noun noun /;
@poschoices = qw/
noun	
adj	
verb	
adverb	
preposition	
determiner	
conjunction	
interjection	
sep	
numeral	
pronoun	
undeterminable	
unknown	
aux	
np	
npf	
npff	
npfm	
npfn	
npl	
npo	
ng	
ngc	
ngi	
ngs	
ngo	
no	
nc	
nt	
n	
link	
punc	
/;


@accelerators = ("<Control-Key-Left>", "<Control-Key-Right>", "<Control-Key-o>", "<Control-Key-n>", "<Control-Key-i>", "<Control-Key-Next>", "<Control-Key-Prior>", "<Control-Key-s>", "<Control-Key-e>", "<Control-Key-z>", "<Control-Key-r>", );

@postlabels = qw/org corr dec trk ortlang comment source convtrk toplinetrk lhpos par extras comp-dev orgmorph freq phontrk normtrk redtrk info comment2 id/;
@posttypes = qw/entry entry entry entry combo entry entry entry entry combo comb entry entry entry entry entry entry entry entry entry entry/;
@postchoices = (0, 0, 0, 0, ['nor.NOR','eng.WWW','fre.WWW','ger.WWW','for.XYZ'], 0, 0, 0, 0,\@poschoices, \@paradigms, 0, 0, 0, 0, 0, 0, 0, 0, 0);

@undo = ();
@redo = ();

$comment = "";
$commString = "Post is changed.\n\nPlease add comment.";
$btnwidth = 8;
$smallbtnwidth = 4;
$currpost = 0;
$d=cwd();

$btnfont = "Arial 11 bold";
$menuitemfont = "Arial 10 normal";
$editfont = "Arial 12 normal";
$changed = 0;  #  Flag to mark if the lexicon data has been changed
$parseSt = 1; #  Variabel som styrer om sofias parser skal brukast

###########################
#                         #
#                         #
#       GUI SETUP         #
#                         #
#                         #
###########################


$mw = MainWindow->new;
$mw->title("GUI shell to Norwegian version 3 of VIPS");
$mw->protocol(WM_DELETE_WINDOW, \&exit_prog);
$scrname = $mw->screen();
$scrh = $mw->screenheight();
$scrh2= $scrh-60;
$scrw = $mw->screenwidth();
print "SCRH=$scrh\nSCRW=$scrw\n";
$mw->geometry($scrw."x".$scrh2."+0+0");
$mw->configure(
#	-height => $scrh-55,
#	-width => $scrw,
	-background => 'black',
	);

#$rulefile = $mw->getOpenFile(-initialdir=>$d,-filetypes => 	[
#	      													['rule files', ['.rule']],
#	      													['All files', ['*.*']]
#	      													]);

#$im = $mw->Photo(-file => "logo.gif");
$im = $mw->Photo(-file => "NST_logo_black.gif");


$dbox = $mw->DialogBox(
#	    -text=>'Les leksikon fra fil?',
		-title => 'Set flags and select source',
	    -default_button=>'Fra fil',
	    -buttons=>['From file', 'Manually']
	   );

$imlbl = $dbox->add(
		'Label',
		-image => $im,
#		-bd => 2,
#		-relief => "ridge",
#		-bg => "lightgray",
		)->pack(ipadx => 20, pady =>20, -anchor => "n");


$diafrm = $dbox->add(
		'Frame',
		-label => "Choose language\n(doesn't do anything yet...)",
#		-bd => 2,
#		-relief => "ridge",
#		-bg => "lightgray",
		)->pack(ipadx => 20, pady =>20, -anchor => "w");


$diarbtn00 = $diafrm->Radiobutton(
#		'Radiobutton',
		-font => 'Arial 14 normal',
		-text => "Swedish",
		-value => "sw",
		-variable => \$currlang,
#		-selectcolor => 'red',
#		-indicatoron => 0,
		)->pack(ipadx => 20, ipady =>0, -anchor => "w");

$diarbtn01 = $diafrm->Radiobutton(
#		'Radiobutton',
		-font => 'Arial 14 normal',
		-text => "Norwegian",
		-value => "no",
		-variable => \$currlang,
#		-selectcolor => 'red',
#		-indicatoron => 0,
		)->pack(ipadx => 20, ipady =>0, -anchor => "w");

$diarbtn02 = $diafrm->Radiobutton(
#		'Radiobutton',
		-font => 'Arial 14 normal',
		-text => "Danish",
		-value => "da",
		-variable => \$currlang,
		-state => "disabled",
#		-selectcolor => 'red',
#		-indicatoron => 0,
		)->pack(ipadx => 20, ipady =>0, -anchor => "w");


$diafrm2 = $dbox->add(
		'Frame',
		-label => "Set flags",
#		-bd => 2,
#		-relief => "ridge",
#		-bg => "lightgray",
		)->pack(ipadx => 20, pady =>20, -anchor => "w");


$diachbtn = $diafrm2->Checkbutton(
#		'Checkbutton',
		-font => 'Arial 14 normal',
		-text => "Print log",
		-variable => \$printlog,
		)->pack(ipadx => 20, ipady =>0, -anchor => "w");

$diachbtn2 = $diafrm2->Checkbutton(
#		'Checkbutton',
		-font => 'Arial 14 normal',
		-text => "Print stems",
		-variable => \$printstems,
		)->pack(ipadx => 20, ipady =>0, -anchor => "w");


$dialbl = $dbox->add(
		'Label',
		-font => 'Arial 14 normal',
		-text => "Please choose input\nfrom lexicon file\nor input manually.",
		)->pack(ipadx => 20, ipady =>20);

$ans = $dbox->Show();

if ($ans eq 'From file') {
    $lexiconfile = $mw->getOpenFile(-initialdir=>$d,-filetypes => $filetypes);
    $job = (split /\./,(split /\//,$lexiconfile)[-1])[0];
#    $job = (split /\//,$lexiconfile)[-1];
    @lexlista = &init_globals();
}


open STEMS, ">GUI_shell.STEMS" if $printstems;
open LOG,">GUI_shell.LOG" if $printlog;
print LOG "Calling sub MkRuleHash($rulefile) returning a hash with paradigm name as key and subrule lines as values\n\n" if $printlog;
#print LOG "Opening a output file \'VIPS.OUT\'\n\n" if $printlog;





$statmsgout = "Rulefile:  $rulefile, Lexiconfile:  $lexiconfile,    Job: $job";
# Bind keyboard accelerators
foreach (@accelerators) {
    $mw->bind($_, [\&key_handler, Ev('K')]);
}
# Create necessary widgets

# Menu frame at top of window for input and interaction
$menuFrame = $mw->Frame(
	-background => 'LightSkyBlue2',
	-takefocus => 0,
	-height => 50,
	-relief => "ridge",
	-borderwidth => 2,
	)->pack(
		-side => 'top',
		-fill => 'x',
		-anchor => 'w',
		);



$mBtn1 = $menuFrame->Menubutton(
	-text => "File",
	-takefocus => 0,
	-font => $btnfont,
	)->pack(
		-side => "left",
		-padx => 1,
		);

$mnu1 = $mBtn1->Menu(
	-font => $menuitemfont,
	-takefocus => 0,
	-tearoff => 0,
	-menuitems => [
			['command' => "Open", -command => \&open_file, -font => $menuitemfont],
			['command' => "Save", -command => \&save_file, -font => $menuitemfont], 
			"-",
			['command' => "Exit", -command => \&exit_prog, -font => $menuitemfont],
		       ],
		     );
	
$mBtn1->configure(-menu => $mnu1);




$mBtn3 = $menuFrame->Menubutton(
	-text => "Settings",
	-takefocus => 0,
	-font => $btnfont,
	)->pack(
		-side => "left",
		-padx => 1,
		);

$mnu3 = $mBtn3->Menu(
	-font => $menuitemfont,
	-takefocus => 0,
	-tearoff => 0,
	-menuitems => [
			['radiobutton' => "Editable text", -variable => \$locktext, -value => 0, -command => sub {$t->configure(-state => "normal")}, -indicatoron => 1 , -font => $menuitemfont],
			['radiobutton' => "Readonly text", -variable => \$locktext, -value => 1, -command => sub {$t->configure(-state => "disabled")}, -indicatoron => 1 , -font => $menuitemfont],
			"-",
			['command' => "Show entire post", -command => \&mk_tl1],

#			['checkbutton' => "Editable text", -variable => \$locktext, -command => sub {$t->configure(-state => "disabled") if $locktext;$t->configure(-state => "normal") unless $locktext}, -indicatoron => 1 , -font => $menuitemfont],
#			['command' => "Open", -command => \&open_file, -font => $menuitemfont],
#			['command' => "Save", -command => \&save_file, -font => $menuitemfont], 
#			"-",
#			['command' => "Exit", -command => \&exit_prog, -font => $menuitemfont],
		       ],
		     );
	
$mBtn3->configure(-menu => $mnu3);




$mBtn2 = $menuFrame->Menubutton(
	-text => "Help",
	-takefocus => 0,
	-font => $btnfont,
	)->pack(
		-side => "right",
		-padx => 1,
		);

$mnu2 = $mBtn2->Menu(
	-font => $menuitemfont,
	-takefocus => 0,
	-tearoff => 0,
	-menuitems => [
			['command' => "Help", -command => [\&help_window, $mw], -font => $menuitemfont],
		       ]
		     );
	
$mBtn2->configure(-menu => $mnu2);

# Frame for showing postFrame and buttons
$post_and_buttonFrame = $mw->Frame(
	-takefocus => 0,
	-height => 50,
	-relief => "ridge",
	-borderwidth => 2,
	)->pack(
		-side => 'top',
		-fill => 'x',
		-anchor => 'w',
		);


# Frame for showing post contents
$postFrame = $post_and_buttonFrame->Frame(
	-background => 'LightSkyBlue1',
	-takefocus => 0,
	-height => 60,
	-relief => "ridge",
	-borderwidth => 2,
	-width => 250,
	)->pack(
		-side => 'left',
		-fill => 'x',
		-anchor => 'w',
		-expand => 1,
		);


$postTitleFrame = $postFrame->Frame(
	-takefocus => 0,
	-relief => "flat",
        -borderwidth => 0,
	-takefocus => 0,
        )->pack(
		-side => 'top',
		-fill => 'x',
		-anchor => 'w',
		);
$postTitleFrame->Label(
	-text => "Contents of post",
	-width => 12,
	-justify => "right",
	-anchor => "e",
	-takefocus => 0,
	)->pack(
		-side => "left",
		-anchor => "w",
		);

$postTitleFrame->Button(
	-width => 10,
	-text => "Clear strings",
	-font => "Arial 10 bold",
	-cursor => hand2,
	-takefocus => 0,
	-relief => 'raised',
	-command => \&clear_strings,
	)->pack(
		-side => 'right',
		-fill => 'none',
		-padx => 10,
		-pady => 3,
		);

$postTitleFrame->Label(
	-text => "Search-strings",
	-width => 12,
	-justify => "right",
	-anchor => "e",
	-takefocus => 0,
	)->pack(
		-side => "right",
		-anchor => "e",
		);
	
	
#SWEDISH   foreach $colnr (1,2,3,4,5,9,10,12,19,20) {	
foreach $colnr (1,2,3,5,10,19,20) {	
	$labentryFrame[$colnr] = $postFrame->Frame(
		-background => 'LightSkyBlue1',
#		-takefocus => 1,
		-bd => 0,
		-relief => "flat",
#		-borderwidth => 5,
		-takefocus => 0,
		)->pack(
			-side => 'top',
			-fill => 'x',
			-anchor => 'w',
			-expand => 1,
			);

	$labentryFrame[$colnr]->Label(
		-background => 'LightSkyBlue2',
		-text => $postlabels[$colnr],
		-width => 10,
		-justify => "right",
		-anchor => "e",
		-takefocus => 0,
		)->pack(
			-side => "left",
			-anchor => "w",
			);
	
	if ($posttypes[$colnr] eq "entry") {
	    $postEntry[$colnr] = $labentryFrame[$colnr]->Entry(
		    -relief => "sunken",
			-takefocus => 0,
		    -foreground => "blue",
			-background => 'LightSkyBlue3',
		    -textvariable => \$posttext[$colnr],
#		    -width => 40,
		    )->pack(
			    -side => "left",
			    -anchor => "w",
			    -fill => 'x',
			    -expand => 1,
			    );	
	}
	else {
		$postEntry[$colnr] = $labentryFrame[$colnr]->BrowseEntryLW(
			-relief => "sunken",
#			-label => $postlabels[$colnr],
			-foreground => "blue",
#			-bd => 4,
			-takefocus => 0,
			-background => 'LightSkyBlue3',
			-variable => \$posttext[$colnr],
#			-width => 37,
			-choices => $postchoices[$colnr],
			)->pack(
				-side => "left",
				-anchor => "w",
				-fill => 'x',
				-expand => 1,
				);	
	    
	}
	$labentryFrame[$colnr]->Entry(
		-relief => "sunken",
		-takefocus => 0,
		-foreground => "blue",
		-background => "white",
		-textvariable => \$searchtext[$colnr],
		-width => 20,
		)->pack(
			-side => "left",
			-anchor => "w",
			);	
}  #end foreach
$postEntry[10]->bind("<FocusOut>", \&new_par);


# A buttonframe..
$buttonFrame = $post_and_buttonFrame->Frame(
	-takefocus => 1,
	-bg => 'black',
	-height => 50,
	-relief => "ridge",
	-borderwidth => 2,
	)->pack(
		-side => 'top',
		-fill => 'x',
		-anchor => 'w',
		);

$recScale = $buttonFrame->Scale(
	-from => 0,
	-to => (scalar(@lexlista) -1),
	-orient => "horizontal",
	-troughcolor => 'grey50',
	-background => 'grey20',
	-resolution => 1,
	-width => 7,
	-showvalue => 0,
	-bd => 1,
	-variable => \$currpost,
	-command => [\&curr_entry_command, 0],
	-bigincrement => 1000,
	)->pack(-side => 'bottom',
			-fill =>'x', 
			pady => 2,
			-expand => 1,
			anchor => 's');



#...with buttons...
$buttonFrame->Button(
	-width => $smallbtnwidth,
	-text => "-10",
	-background => 'grey50',
	-font => "Arial 10 bold",
	-cursor => hand2,
	-takefocus => 0,
	-relief => 'raised',
	-command => [\&curr_entry_command, -10],
	)->pack(
		-side => 'left',
		-fill => 'none',
		-padx => 2,
		-pady => 1,
		);

$buttonFrame->Button(
	-width => $smallbtnwidth,
	-text => "-1",
	-background => 'grey50',
	-font => "Arial 10 bold",
	-cursor => hand2,
	-takefocus => 0,
	-relief => 'raised',
	-command => [\&curr_entry_command, -1],
	)->pack(
		-side => 'left',
		-fill => 'none',
		-padx => 2,
		-pady => 1,
		);

$gotoEntry = $buttonFrame->Entry(
	-background => 'grey70',
	-width => 7,
	-textvariable => \$currpost,
	)->pack(
		-side => 'left',
		-fill => 'none',
		-padx => 2,
		-pady => 1,
		);

$gotoEntry->bind("<Return>",\&curr_entry_command);
		
$buttonFrame->Button(
	-width => $smallbtnwidth,
	-text => ">",
	-background => 'grey50',
	-font => "Arial 8 bold",
	-takefocus => 0,
	-cursor => hand2,
	-relief => 'raised',
	-command => [\&curr_entry_command, 1],
	)->pack(	-side => 'left',
		-fill => 'none',
		-padx => 2,
		-pady => 1,
		);

$buttonFrame->Button(
	-width => $smallbtnwidth,
	-height => 1,
	-background => 'grey50',
	-text => ">>",
	-font => "Arial 8 bold",
	-takefocus => 0,
	-cursor => hand2,
	-relief => 'raised',
	-command => [\&curr_entry_command, 10],
	)->pack(-side => 'left',
		-fill => 'x',
		-padx => 2,
		-pady => 1,
		);


#  Edit-frame
$editFrame = $post_and_buttonFrame->Frame(
	-takefocus => 0,
	-height => 50,
	-relief => "ridge",
	-background => "DarkSeaGreen4",
	-borderwidth => 2,
	)->pack(
		-side => 'top',
		-fill => 'x',
		-anchor => 'w',
		);


$editOrt = $editFrame->Entry(
		-takefocus => 1,
		-font => $editfont,
		-relief => "flat",
		-foreground => "blue",
		-background => "DarkSeaGreen2",
		-textvariable => \$posttext[1],
		-width => 30,
		);

$editFrame->Label(
		-takefocus => 0,
		-font => $editfont,
		-background => "DarkSeaGreen4",
		-text => $postlabels[1],
		-width => 10,
		-justify => "right",
		-anchor => "e",
		-takefocus => 0,
		  )->grid($editOrt, '-',  -sticky => "w");
	
$parseTrk = $editFrame->Checkbutton(
		-takefocus => 0,
		-background => "DarkSeaGreen4",
		-font => $menuitemfont,
	        -text => "Check transcription",
                -variable => \$parseSt
		);

$editTrk = $editFrame->Entry(
		-takefocus => 1,
		-font => $editfont,
		-relief => "flat",
		-background => "DarkSeaGreen2",
		-foreground => "blue",
		-textvariable => \$posttext[3],
		-width => 30,
		);
$editTrk->bind("<FocusOut>", \&parse);

$editFrame->Label(
		-font => $editfont,
		-background => "DarkSeaGreen4",
		-text => $postlabels[3],
		-width => 10,
		-justify => "right",
		-anchor => "e",
		-takefocus => 0,
		  )->grid($editTrk, $parseTrk, -sticky => "w");

$editComm = $editFrame->Entry(
		-takefocus => 1,
		-font => $editfont,
		-relief => "flat",
		-foreground => "blue",
		-background => "DarkSeaGreen2",
		-textvariable => \$posttext[19],
		-width => 30,
		);

$commMenu = $editFrame->Optionmenu(
		-takefocus => 1,
		-background => "DarkSeaGreen3",
#		-foreground => "DarkSeaGreen3",
		-font => $editfont,
		-width => 20,
		-options => \@defComm,
	        -textvariable => \$comment,
		-command => \&append_comment
		);

$editFrame->Label(
		-font => $editfont,
		-background => "DarkSeaGreen4",
		-text => $postlabels[5],
		-width => 10,
		-justify => "right",
		-anchor => "e",
		-takefocus => 0,
		 )->grid($editComm, $commMenu);
	
$editPar = $editFrame->MultiMenu(
#		-relief => "flat",
		-takefocus => 1,
		-background => "DarkSeaGreen2",
		-foreground => "red",
		-font => $editfont,
		-width => 20,
		-choices => \@paradigms,
		-variable => \$posttext[10],
		-listcmd => \&new_par
                );

$editFrame->Label(
		-font => $editfont,
		-background => "DarkSeaGreen4",
		-text => $postlabels[10],
		-width => 10,
		-justify => "right",
		-anchor => "e",
		-takefocus => 0,
		  )->grid($editPar, -sticky => "w");


$btn2Frame = $post_and_buttonFrame->Frame(
	-takefocus => 0,
	-height => 10,
	-background => "DarkSlateGray4",
	-relief => "ridge",
	-borderwidth => 2,
	)->pack(
		-side => 'top',
		-fill => 'x',
		-anchor => 'w',
		);


$iallBtn = $btn2Frame->Button(
		-takefocus => 0,
	-background => "DarkSlateGray3",
		   -font => $btnfont,
		   -text => "Inflect all",
		   -width => 10,
		   -command => \&inflect_all
		   );    
$okBtn = $btn2Frame->Button(
		-takefocus => 0,
	-background => "DarkSlateGray2",
		   -font => $btnfont,
		   -text => "Ok",
		   -width => 10,
		   -command => \&set_ok
		   );
$regelBtn = $btn2Frame->Button(
		-takefocus => 0,
	-background => "DarkSlateGray2",
		   -font => $btnfont,
		   -text => "Rule error",
		   -width => 10,
		   -command => \&set_regelfel
		   );
$reviewBtn = $btn2Frame->Button(
		-takefocus => 0,
	-background => "DarkSlateGray2",
		   -font => $btnfont,
		   -text => "Review",
		   -width => 10,
		   -command => \&set_review
		   );
$nextBtn = $btn2Frame->Button(
			-takefocus => 0,
	-background => "DarkSlateGray3",
		   -font => $btnfont,
		   -text => "Next",
		   -width => 10,
		   -command => [\&new_record, 1]
		   );
$prevBtn = $btn2Frame->Button(
			-takefocus => 0,
	-background => "DarkSlateGray3",
			-font => $btnfont,
			-text => "Previous",
		   -width => 10,
		   -command => [\&new_record, -1]
		   );
$btn2Frame->Button(
		   	-takefocus => 0,
			-background => "DarkSlateGray3",
			-font => $btnfont,
		   -text => "Inflect",
		   -width => 10,
		   -command => \&inflect
		   )->grid($iallBtn, $okBtn, $regelBtn, $reviewBtn,$prevBtn, $nextBtn, -sticky => "w", -padx => 1, -pady => 2,);
		  
# A status bar at the bottom
$statusFrame = $mw->Frame(
	-borderwidth => 3,
	-takefocus => 0,
	)->pack(
		-side => 'bottom',
		-fill => 'x',
		);

$ruleFrame = $statusFrame->Frame(
	-borderwidth => 2,
	-relief => "ridge",
	-takefocus => 0,
	)->pack(
		-side => 'left',
		-fill => 'x',
		);
$lexFrame = $statusFrame->Frame(
	-borderwidth => 2,
	-relief => "ridge",
	-takefocus => 0,
	)->pack(
		-side => 'left',
		-fill => 'x',
		);


# With a label
$ruleFrame->Label(
	-textvariable => \$statmsgout,
	-borderwidth =>2,
	-font => "Arial 10",
	-takefocus => 0,
	)->pack(
		-side => 'left',
		-fill => 'x',
		-anchor => 'w'
		);
$lexFrame->Label(
	-textvariable => \$statmsgout,
	-borderwidth =>2,
	-font => "Arial 10",
	-takefocus => 0,
	)->pack(
		-side => 'left',
		-fill => 'x',
		-anchor => 'w'
		);

# A text area above the status bar
$t = $mw->Scrolled("Text",
#	-background => "green",
	-tabs => [qw/8c 16c 24c 32c/],
	-height => 10,
	-takefocus => 0,
	-font => "Courier 10",
	-wrap => "none",
	-scrollbars => 'se'
	)->pack(
		-side => 'bottom',
		-fill => 'both',
		-expand => 1,
		);
$t->tagConfigure('red', -foreground => "red");
$t->tagConfigure('blue', -foreground => "blue");
$t->tagConfigure('black', -foreground => "black");
print LOG "\nPOSTTEXT:   @posttext[1,3,10,2]\n\n";
$vipsreturn = &vips(@posttext[1,3,10,2]);
#$vipsreturn = &vips(@posttext);
$t->insert("1.0",$vipsreturn."\n");



###########################################
@colornames = qw/gray60 gray70 gray80 gray85 gray90
gray95 snow1 snow2 snow3 snow4 seashell1 seashell2 seashell3 seashell4
AntiqueWhite1 AntiqueWhite2 AntiqueWhite3 AntiqueWhite4 bisque1
bisque2 bisque3 bisque4 PeachPuff1 PeachPuff2 PeachPuff3 PeachPuff4
NavajoWhite1 NavajoWhite2 NavajoWhite3 NavajoWhite4 LemonChiffon1
LemonChiffon2 LemonChiffon3 LemonChiffon4 cornsilk1 cornsilk2
cornsilk3 cornsilk4 ivory1 ivory2 ivory3 ivory4 honeydew1 honeydew2
honeydew3 honeydew4 LavenderBlush1 LavenderBlush2 LavenderBlush3
LavenderBlush4 MistyRose1 MistyRose2 MistyRose3 MistyRose4 azure1
azure2 azure3 azure4 SlateBlue1 SlateBlue2 SlateBlue3 SlateBlue4
RoyalBlue1 RoyalBlue2 RoyalBlue3 RoyalBlue4 blue1 blue2 blue3 blue4
DodgerBlue1 DodgerBlue2 DodgerBlue3 DodgerBlue4 SteelBlue1 SteelBlue2
SteelBlue3 SteelBlue4 DeepSkyBlue1 DeepSkyBlue2 DeepSkyBlue3
DeepSkyBlue4 SkyBlue1 SkyBlue2 SkyBlue3 SkyBlue4 LightSkyBlue1
LightSkyBlue2 LightSkyBlue3 LightSkyBlue4 SlateGray1 SlateGray2
SlateGray3 SlateGray4 LightSteelBlue1 LightSteelBlue2 LightSteelBlue3
LightSteelBlue4 LightBlue1 LightBlue2 LightBlue3 LightBlue4 LightCyan1
LightCyan2 LightCyan3 LightCyan4 PaleTurquoise1 PaleTurquoise2
PaleTurquoise3 PaleTurquoise4 CadetBlue1 CadetBlue2 CadetBlue3
CadetBlue4 turquoise1 turquoise2 turquoise3 turquoise4 cyan1 cyan2
cyan3 cyan4 DarkSlateGray1 DarkSlateGray2 DarkSlateGray3
DarkSlateGray4 aquamarine1 aquamarine2 aquamarine3 aquamarine4
DarkSeaGreen1 DarkSeaGreen2 DarkSeaGreen3 DarkSeaGreen4 SeaGreen1
SeaGreen2 SeaGreen3 SeaGreen4 PaleGreen1 PaleGreen2 PaleGreen3
PaleGreen4 SpringGreen1 SpringGreen2 SpringGreen3 SpringGreen4 green1
green2 green3 green4 chartreuse1 chartreuse2 chartreuse3 chartreuse4
OliveDrab1 OliveDrab2 OliveDrab3 OliveDrab4 DarkOliveGreen1
DarkOliveGreen2 DarkOliveGreen3 DarkOliveGreen4 khaki1 khaki2 khaki3
khaki4 LightGoldenrod1 LightGoldenrod2 LightGoldenrod3 LightGoldenrod4
LightYellow1 LightYellow2 LightYellow3 LightYellow4 yellow1 yellow2
yellow3 yellow4 gold1 gold2 gold3 gold4 goldenrod1 goldenrod2
goldenrod3 goldenrod4 DarkGoldenrod1 DarkGoldenrod2 DarkGoldenrod3
DarkGoldenrod4 RosyBrown1 RosyBrown2 RosyBrown3 RosyBrown4 IndianRed1
IndianRed2 IndianRed3 IndianRed4 sienna1 sienna2 sienna3 sienna4
burlywood1 burlywood2 burlywood3 burlywood4 wheat1 wheat2 wheat3
wheat4 tan1 tan2 tan3 tan4 chocolate1 chocolate2 chocolate3 chocolate4
firebrick1 firebrick2 firebrick3 firebrick4 brown1 brown2 brown3
brown4 salmon1 salmon2 salmon3 salmon4 LightSalmon1 LightSalmon2
LightSalmon3 LightSalmon4 orange1 orange2 orange3 orange4 DarkOrange1
DarkOrange2 DarkOrange3 DarkOrange4 coral1 coral2 coral3 coral4
tomato1 tomato2 tomato3 tomato4 OrangeRed1 OrangeRed2 OrangeRed3
OrangeRed4 red1 red2 red3 red4 DeepPink1 DeepPink2 DeepPink3 DeepPink4
HotPink1 HotPink2 HotPink3 HotPink4 pink1 pink2 pink3 pink4 LightPink1
LightPink2 LightPink3 LightPink4 PaleVioletRed1 PaleVioletRed2
PaleVioletRed3 PaleVioletRed4 maroon1 maroon2 maroon3 maroon4
VioletRed1 VioletRed2 VioletRed3 VioletRed4 magenta1 magenta2 magenta3
magenta4 orchid1 orchid2 orchid3 orchid4 plum1 plum2 plum3 plum4
MediumOrchid1 MediumOrchid2 MediumOrchid3 MediumOrchid4 DarkOrchid1
DarkOrchid2 DarkOrchid3 DarkOrchid4 purple1 purple2 purple3 purple4
MediumPurple1 MediumPurple2 MediumPurple3 MediumPurple4 thistle1
thistle2 thistle3 thistle4/;

$t->tagConfigure('black', -foreground => "black");


foreach $cn (@colornames) {$t->tagConfigure($cn, -foreground => $cn)}
#foreach $cn (@colornames) {$t->insert("end",$cn."\n",$cn)}


###########################################




$menuFrame->focus();

$changed = 0;



MainLoop;

#############################################################################
#
#  CALLBACK-FUNCTIONS
#
#############################################################################

sub exit_prog {
	if ($changed == 1 || &check_change_and_update()) {
		$ans = &ask_save();
		if ($ans eq "Ja") {
			&save_file();
		}
		elsif($ans eq "Avbryt") {
			return;
		}
	}
	close LOG if $printlog;
	close STEMS if $printstems;
    exit;
}

sub open_file {
    if ($changed == 1 || &check_change_and_update()) {
	$ans = &ask_save();
	if ($ans eq "Ja") {
	    &save_file();
	}
	elsif($ans eq "Avbryt") {
	    return;
	}
    }
    $lexiconfile = $mw->getOpenFile(-initialdir=>$d,-filetypes => $filetypes);
    $job = (split /\./,(split /\//,$lexiconfile)[-1])[0];
#    $job = (split /\//,$lexiconfile)[-1];
    @lexlista = &init_globals();
    $recScale->configure(-to => (scalar @lexlista - 1)); 
    $recScale->configure(-from => 0);
    $currpost = 0;
    &curr_entry_command(0);
	$statmsgout = "Rulefile:  $rulefile, Lexiconfile:  $lexiconfile,    Job: $job";
}

sub save_file {
    &update_post();
    open OUTFILE, ">$lexiconfile" || die "Kan ikkje opne fil: $lexiconfile, $!";
    foreach (@lexlista) {
	print OUTFILE "$_\n";
    }
    $changed = 0;
    close OUTFILE;
}

sub inflect {
    $t->configure(-state => "normal");
    $t->delete("1.0", "end");
    $vipsreturn = &vips(@posttext[1,3,10,2]);
#	$vipsreturn = &vips(@posttext);
    $t->insert("1.0",$vipsreturn."\n");
#    foreach $cn (@colornames) {$t->insert("end",$cn."\n",$cn)}
    $t->configure(-state => "disabled") if $locktext;
}

sub inflect_all {
    my(@post);
    my($s) = 1;
    my($count) = -1;
    my($inflData);
    $filename = $mw->getSaveFile(-initialdir=>$d);
    if ($filename ne "") {
	open OUTFILE, ">$filename" || &msg_box("Kan ikkje opne fil, $filename,\n$!");
	do {
	    $count+=$s;
	    @post = split("\t", $lexlista[$count]);
	    $inflData = &vips(@post[1,3,10,2]);
#	    $inflData = &vips(@post);
	    print OUTFILE "$inflData\n";
	    print OUTFILE "------------------------------------------------------------------------\n";
	}
	while ($s = &find_next(1, $count));
	close OUTFILE;
    }
}
	    

sub parse {
    if ($parseSt) {
	my(@v) = ();
	my($asc) = "";
	$v[2] = $posttext[2];
	$v[3] = $posttext[9];
	$v[6] = $posttext[4];
	$v[11] = $posttext[3];
	$v[40] = $posttext[20];
	foreach (split("", join(";", @v))) {
	    $asc .= ord($_) . "_";
	}
	chop($asc);  
	$res = `perl newReadParser_swe.pl $asc`; #  Kaller modifisert utgave av sofias parser (modifisert til  dekoda input-argumentet
	if ($res ne "") {
	    &msg_box($res);
	}
    }
}

sub new_par {
    if ($old_par ne $posttext[10]) {
	push(@undo, $old_par);
	$old_par = $posttext[10];
    }
}

sub undo_par {
    if (scalar @undo > 0) {
	push(@redo, $posttext[10]);
	$posttext[10] = pop(@undo);
	#$parVar = $posttext[10];
	$old_par = $posttext[10];
    }
}

sub redo_par {
    if (scalar @redo > 0) {
	push(@undo, $posttext[10]);
	$posttext[10] = pop(@redo);
	#$parVar = $posttext[10];
	$old_par = $posttext[10];
    }
}


sub append_comment {
    if ($posttext[19]!~/\<$comment\>/) {
	$posttext[19] .= "<$comment\($job\)>";
    }
}

sub set_ok {
    my($c) = "<status=OK($job)>";
    if ($posttext[19]!~/$c/) {
	$posttext[19] .= $c;
    }
    &new_record(1);
}

sub set_review {
    my($c) = "<status=NOT_OK_review($job)>";
    if ($posttext[19]!~/$c/) {
	$posttext[19] .= $c;
    }
    &new_record(1);
}

sub set_regelfel {
    my($c) = "<status=NOT_OK_regelfel($job)>";
    if ($posttext[19]!~/$c/) {
	$posttext[19] .= $c;
    }
    &new_record(1);
}


sub key_handler {
    my($key) = $_[1];
    if ($key eq "Left")  {
	&curr_entry_command(-1);
    }
    elsif ($key eq "Right") {
	&curr_entry_command(1);
    }
    elsif ($key eq "Prior") {
	&curr_entry_command(-10);
    }
    elsif ($key eq "Next") {
	&curr_entry_command(10);
    }
    elsif ($key eq "n") {
	&new_record(1);
    }
    elsif ($key eq "o") {
	&set_ok();
    }
    elsif ($key eq "i") {
	&inflect();
    }
    elsif ($key eq "s") {
	&save_file();
    }
    elsif ($key eq "e") {
	&exit_prog();
    }
    elsif ($key eq "z") {
	&undo_par();
    }
    elsif ($key eq "r") {
	&redo_par();
    }

}

#  Skriv kommentar ang postChanged
sub new_record {
    if(&check_change() && $posttext[19]!~/status/) {
	$ans = $mw->Dialog(
			   -text => $commString,
			   -default_button => "Ok",
			   -buttons => ["Ok"]
			   )->Show;
	return;
    }   
    &update_post();
    #  Finn neste post som passer skekriteria
    &curr_entry_command(&find_next($_[0], $currpost));
}

sub curr_entry_command {
    $comment = "";
    $currpost += $_[0] if (($currpost + $_[0]) <= scalar @lexlista && 
			   ($currpost + $_[0]) >= 0) ;
    &show_nextentries($lexlista[$currpost]);
    $t->configure(-state => "normal");
    $t->delete("1.0", "end") if ($t);
    
    $vipsreturn = &vips(@posttext[1,3,10,2]);
#    $vipsreturn = &vips(@posttext);
    $t->insert("1.0",$vipsreturn."\n") if ($t);
#    foreach $cn (@colornames) {$t->insert("end",$cn."\n",$cn)}    
    $t->configure(-state => "disabled") if $locktext;
    #$parVar = $posttext[10];
}

sub clear_strings {
    for(1..20) {
	$searchtext[$_] = "";
    }
}
	
#############################################################################
#
# HELPER-FUNCTIONS
#
#############################################################################
sub ask_save {
    $ans = $mw->Dialog(
	        -text => "Leksikonfilen er endret.  Vil du lagre endringene?",
		-default_button => "Ja",
		-buttons => ["Ja", "Nei", "Avbryt"]
		       )->Show;
    return $ans;
}

sub msg_box {
    $ans = $mw->Dialog(
		       -text => $_[0],
		       -buttons => ["Ok"]
		       )->Show;
    return 0;
}

sub init_globals {
	my (@lexlist);
	open IN,"$lexiconfile" || die "Can't open file $lexiconfile, $!";
	while (<IN>) {
		chomp;
		push @lexlist,$_ if (/\S/ && $_ ne "");
	} # end while
	close IN;
	return 	@lexlist;
}


sub show_nextentries { #call: &show_nextentries($lexlista[$currpost]);
    my $currlexline = shift;
    my @fields = split /\t/,$currlexline;
    foreach (0..20) {
	$posttext[$_] = $fields[$_];
    }
    undef @undo;
    undef @redo;
    $old_par = $posttext[10];
}

sub check_change_and_update {
    my $n = join("\t", @posttext);
    $n=~s/\s*\Z//;
    if ($n ne $lexlista[$currpost]) {
	$lexlista[$currpost] = $n;
	$changed = 1;
	return 1;
    }
    else {
	return 0;
    }
}

sub check_change {
    my $n = join("\t", @posttext);
    $n=~s/\s*\Z//;
    if ($n ne $lexlista[$currpost]) {
	return 1;
    }
    else {
	return 0;
    }
}

sub update_post {
    my $n = join("\t", @posttext);
    $n=~s/\s*\Z//;

    $lexlista[$currpost] = $n;
    $changed = 1;
}

sub find_next {
    my($step) = $_[0];
    my($curr_pos) = $_[1]; 
    my($found) = 0;
    my(@post) = ();
    my($c) = 0;
    while (1) {
	$match = 1;
	$c = 0;
	$curr_pos += $step;
	@post = split("\t", $lexlista[$curr_pos]);
	until($c > 20) {	    
	    if ($searchtext[$c] ne "" && $post[$c] !~ /$searchtext[$c]/) {
		$match = 0; # Feltet stemmer ikkje med kriterium, 
		$c = 20;    # g til neste post. 
	    }	    
	    $c++;
	}
	if ($curr_pos < 0 || $curr_pos >= scalar @lexlista) {
	    return 0;
	}
	if ($match == 1) {
	    return ($curr_pos - $_[1]);
	}
    }	    
}














sub mk_tl1 {
	if (! Exists($tl1)) {
		$tl1 = $mw->Toplevel(
			-title => "Entire post content",
#			-width => '300',
#			-height => '400',
#			-background => 'red',
		);
#		$tl1->geometry("400x300+100+50");



foreach $colnr (0..20) {	
	$labentryFrame[$colnr] = $tl1->Frame(
		-background => 'LightSkyBlue1',
#		-takefocus => 1,
		-bd => 0,
		-relief => "flat",
#		-borderwidth => 5,
		-takefocus => 0,
		)->pack(
			-side => 'top',
			-fill => 'x',
			-anchor => 'w',
			-expand => 1,
			);

	$labentryFrame[$colnr]->Label(
		-background => 'LightSkyBlue2',
		-text => $postlabels[$colnr],
		-width => 10,
		-justify => "right",
		-anchor => "e",
		-takefocus => 0,
		)->pack(
			-side => "left",
			-anchor => "w",
			);
	
	if ($posttypes[$colnr] eq "entry") {
	    $postEntry[$colnr] = $labentryFrame[$colnr]->Entry(
		    -relief => "sunken",
			-takefocus => 0,
		    -foreground => "blue",
			-background => 'LightSkyBlue3',
		    -textvariable => \$posttext[$colnr],
		    -font => "Courier 14 bold",
#		    -width => 40,
		    )->pack(
			    -side => "left",
			    -anchor => "w",
			    -fill => 'x',
			    -expand => 1,
			    );	
	}
	else {
		$postEntry[$colnr] = $labentryFrame[$colnr]->BrowseEntryLW(
			-relief => "sunken",
#			-label => $postlabels[$colnr],
			-foreground => "blue",
#			-bd => 4,
			-takefocus => 0,
			-background => 'LightSkyBlue3',
			-variable => \$posttext[$colnr],
			-font => "Arial 12 normal",
#			-width => 37,
			-choices => $postchoices[$colnr],
			)->pack(
				-side => "left",
				-anchor => "w",
				-fill => 'x',
				-expand => 1,
				);	
	$swdgt[$colnr] = $postEntry[$colnr]->Subwidget('slistbox');
	$swdgt[$colnr]->configure(-height => 15);
	    
	}
}  #end foreach
$postEntry[10]->bind("<FocusOut>", \&new_par);







	} # end if
	else {
		$tl1->deiconify();
		$tl1->raise;
	} # end else
} # end sub

