#!perl -w

## Filename:	swe_lexicon.pl
## Author:		Sofia Strmbergsson
## Created:		000228
##
## Description:	Lexicon used by Swedish parser.
##

###################################################
# LANGUAGE INDEPENDENT LEXICON
###################################################
$HEAD_CHARS = '[a-zA-EG-JL-Z\"\#\$\'\.7-9\?\@\^\_\*]';
$TAIL_CHARS = '[!-+0-6:<>FK\\{}\`~]';
$DIACS_STRUCTURE = '(%[^"]|%"[^"]+")';


###################################################
# LANGUAGE SPECIFIC LEXICON
###################################################
$SWE_SYMBOL = 'a&U|e&U|i:|I|y:|Y|e\+:|e\+|\$|e:|e|E:|E|\@|\@:|A:|a|u0:|o0|u:|U|o:|O|'.
	 'p|b|t\+?|d\+?|k|g|m|n\+?|nK|f|V|s\+?|r|C|h|h6~|j|l\+?|2\'|2\"|\'2|\'|\"|\_|\.';
$SWE_VOWEL = 'i:|I|y:|Y|e\+:|e\+|\$|e:|e|E:|E|\@|\@:|A:|a|u0:|o0|u:|U|o:|O|a&U|e&U';
$SWE_LONG_VOWEL = 'i:|y:|e\+:|\e:|E:|\@:|A:|u0:|u:|o:';
$SWE_SHORT_VOWEL = 'I|Y|e\+|\$|e|E|\@|a|o0|U|O';
$SWE_DIPHTHONG = 'a&U|e&U';			# if none, replace with 'NULL'
$SWE_CONSONANT = 'p|b|t\+?|d\+?|k|g|m|nK|n\+?|f|V|s\+?|r|C|h|h6~|j|l\+?';
$SWE_NOT_SOUND = '2|\'|\"|\.|\_';
$SWE_MAIN_STRESS_MARK = '2?[\'\"][^2]';		# if none, replace with 'NULL'
$SWE_SEC_STRESS_MARK = '\'2';			# if none, replace with 'NULL'
$SWE_SYLLABLE_DELIMITER = '\.|\_';
$SWE_WORD_DELIMITER = '\_';
$SWE_SYLLABIFIED_CONS = 'NULL';		# in eg Norwegian: 'n%\)'
$SWE_COMPOUND_MARKER = '\+';

$SWE_TONEME1 = "([^\']+($SWE_SYLLABLE_DELIMITER)|)\'[^\'2]+";		# if none, replace with 'NULL'
$SWE_TONEME2 = "([^\']+($SWE_SYLLABLE_DELIMITER)|)2\'[^2\']+(\.\'2)?[^2\']+";	# if none, replace with 'NULL'
# if none, replace with 'NULL'
$SWE_PHRASE_ACCENT = "(([^\'\"]+($SWE_SYLLABLE_DELIMITER)|)2\"[^2\'\"]+(\.\'2)?[^2\'\"]+)|(([^\'\"]+($SWE_SYLLABLE_DELIMITER)|)\"[^\'2\"]+)";

# if the trc string matches the pattern X, the conditions Y, Z and/or W must be fulfilled, where 
# Y = morphological condition
# Z = condition for expansion field
# W = condition on pos
# AND = all conditions must be fulfilled, OR = at least one of the conditions must be fulfilled, <EMPTY> = no conditions
$SWE_COMB_CONDITION{''} = ";;;";


# $SWE_TRC_IFF_LANG{TRC} = LANG
# if a trc-sequence should only be found in entries where the language code is equal to LANG
$SWE_TRC_IFF_LANG{""} = "";


# Violations generate error messages:
#$SWE_X_CANNOT_FOLLOW_Y{'Y'} = 'X';
#$SWE_X_SHOULD_NOT_FOLLOW_Y{'()()(X1)()'} = '(X2)()(X3)';		X1, X2 and X3 will be printed in error message

# Violations generate warning messages:
#$SWE_X_SHOULD_NOT_FOLLOW_Y{Y} = X;
#$SWE_X_SHOULD_NOT_FOLLOW_Y{()()(X1)()} = (X2)()(X3);		X1, X2 and X3 will be printed in warning message
$SWE_X_SHOULD_NOT_FOLLOW_Y{"()($SWE_LONG_VOWEL)(r)()"} = '()('.$SWE_SYLLABLE_DELIMITER.')*([nsdt]\+?)';			# ej r fre retroflexkandidat (utom l)
#$SWE_X_SHOULD_NOT_FOLLOW_Y{'()()(r)()'} = '()('.$SWE_SYLLABLE_DELIMITER.')*([l])[^\+]';					# ej icke-retroflext l efter r
$SWE_X_SHOULD_NOT_FOLLOW_Y{'()()([lnsdt]\+)()'} = '()('.$SWE_NOT_SOUND.')*([lnsdt])[^\+]';					# ej icke-retroflex retroflexkandidat efter retroflex
$SWE_X_SHOULD_NOT_FOLLOW_Y{"($SWE_SHORT_VOWEL)($SWE_CONSONANT)*([bgV])($SWE_NOT_SOUND)*"} = '()()([ptks])';		# devoicing av /bgV/
$SWE_X_SHOULD_NOT_FOLLOW_Y{"($SWE_SHORT_VOWEL)(p|b|t\+?|d\+?|k|g|m|f|V|s\+?|r|C|h|h6~|j|l\+?)*(d)($SWE_NOT_SOUND)*"} = '()()([ptks])';	# devoicing av /d/
$SWE_X_SHOULD_NOT_FOLLOW_Y{'()()(n)()'} = "($SWE_SYLLABLE_DELIMITER)($SWE_MAIN_STRESS_MARK)*(g)";			# ej ng ver stavelsegrns
# ej /e/ i betonad stavelse fre /r/ eller retroflex (ska vara /E/ eller /e:/)
$SWE_X_SHOULD_NOT_FOLLOW_Y{"()()($SWE_MAIN_STRESS_MARK|$SWE_SEC_STRESS_MARK)($SWE_CONSONANT)*"} = "(e)($SWE_NOT_SOUND)*(r|[lnsdt]\\+)";



#$TRC_X_CORRESPONDS_TO_ORTO_Y{Y} = X;
# in a X-pattern such as /(X)(Y)(Z)/, extract X and Z, in a Y pattern such as /(X)(Y)(Z)/, extract X and Z
$TRC_X_CORRESPONDS_TO_ORTO_Y{'()([^r])(ng)'} = '()()(nK)';						# <[^r]ng> -> /nK/
$TRC_X_CORRESPONDS_TO_ORTO_Y{'()([^r])(n\+g)[^jiey]'} = "(nK)($SWE_NOT_SOUND)*(g)";				# <[^r]n+g> -> /nKg/
$TRC_X_CORRESPONDS_TO_ORTO_Y{'()([^r])(n\+?k)'} = "((nK)($SWE_NOT_SOUND)*(k)|(n)($SWE_NOT_SOUND)*(C))";		# <[^r]n(+)?k> -> /nKk/ eller /nC/
$TRC_X_CORRESPONDS_TO_ORTO_Y{"([^skghtdlaoueiy])()(\\+\\1)"} = "($SWE_CONSONANT)($SWE_NOT_SOUND)+(\\1)";	# samma konsonant p varsin sida om stavelsegrns
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^r]\\+?(s)()(\\+s)[^kjtch]'} = "(s)($SWE_NOT_SOUND)+(s)";			# d:o: s (dr s ej r retroflext)
$TRC_X_CORRESPONDS_TO_ORTO_Y{'r[^dtln]\\+?(s)()(\\+s)[^kjtch]'} = "(s)($SWE_NOT_SOUND)+(s)";			# d:o: s (dr s ej r retroflext)
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^rdtln\\+]\\+?(s)()(\\+s)[t][^j]'} = "(s)($SWE_NOT_SOUND)+(s)";			# d:o: s
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^r\\+]\\+?(s)()(\\+s)[c][^h]'} = "(s)($SWE_NOT_SOUND)+(s)";			# d:o: s
$TRC_X_CORRESPONDS_TO_ORTO_Y{'r[dtl]?\\+?(s)()(\\+s)[^kjtch]'} = "(s\\+)($SWE_NOT_SOUND)+(s\\+)";			# d:o: s+
$TRC_X_CORRESPONDS_TO_ORTO_Y{'r[dtl]?\\+?(s)()(\\+s)[t][^j]'} = "(s\\+)($SWE_NOT_SOUND)+(s\\+)";			# d:o: s+
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^n](g)()(\\+g)[^jiey]'} = "(g|j)($SWE_NOT_SOUND)+(\\1)";			# d:o: g
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^r](t)()(\\+t)[^j]'} = "(t)($SWE_NOT_SOUND)+(t)";				# d:o: t
$TRC_X_CORRESPONDS_TO_ORTO_Y{'[^r](d)()(\\+d)[^j]'} = "(d)($SWE_NOT_SOUND)+(d)";				# d:o: d
$TRC_X_CORRESPONDS_TO_ORTO_Y{'rs?(t)()(\\+t)[^j]'} = "(t\\+)($SWE_NOT_SOUND)+(t\\+)";				# d:o: t+
$TRC_X_CORRESPONDS_TO_ORTO_Y{'r(d)()(\\+d)[^j]'} = "(d\\+)($SWE_NOT_SOUND)+(d\\+)";				# d:o: d+
$TRC_X_CORRESPONDS_TO_ORTO_Y{'(l)()(\\+l)[^j]'} = "(l)($SWE_NOT_SOUND)+(l)";					# d:o: l
$TRC_X_CORRESPONDS_TO_ORTO_Y{'r(l)()(\\+l)[^j]'} = "(l\\+)($SWE_NOT_SOUND)+(l\\+)";				# d:o: l+
$TRC_X_CORRESPONDS_TO_ORTO_Y{'(nds)[^j]()()'} = "(nd)($SWE_NOT_SOUND)*(s)";				# <nds> -> /nds/
# <+s+> -> /(s+?)($SWE_NOT_SOUND)+/, i.e. linking /s/ to the left of the syllable boundary
$TRC_X_CORRESPONDS_TO_ORTO_Y{'(\+s\+)([^s])()'} = "(s\\+?)($SWE_NOT_SOUND)+()";					


# exeptions to the rule that duplication probably is unwanted
$SWE_ALLOWED_DUPLICATION = '';		# if none, replace with ''

# if none, replace with 'NULL'
$SWE_RARE_ONSET = '^(skl|ps|dj|sf)$';

# if none, replace with 'NULL'
$SWE_NOT_INITIAL_ONSET = '^[stln]\+';

$SWE_LEGAL_ONSET = '^(s(\+)?pr|s(\+)?pj|s(\+)?pl|str|s\+t\+r|s(\+)?kr|s(\+)?kV|'.
		 'pr|pj|pl|br|bj|bl|t(\+)?r|t(\+)?V|d(\+)?r|d(\+)?V|kr|kl|kV|kn|'.
		 'gr|gl|gn|fr|fl|fj|fn|Vr|s(\+)?p|st|s\+t\+|s(\+)?k|s(\+)?V|sl|'.
		 's\+l\+|s(\+)?m|sn|s\+n\+|mj|n(\+)?j|p|b|t(\+)?|d(\+)?|k|'.
		 'g|f|V|l(\+)?|r|j|s(\+)?|h6~|C|m|n(\+)?|h)$';	


# if none, replace with 'NULL'
$SWE_RARE_CODA = '^(gms?|j(l|V|n|p)s?|n\+s\+t\+s?|rd|rs\+s?|rt|rt\+|tms?|ts\+)$';

$SWE_LEGAL_CODA = '^(|[^hC]|'.				# empty string or any single character except h and C
		         '(b|d|t|lm|nd)(s|skt?)?|'.		# -s, -sk, -skt
		         '(n|m|f)(s|skt?|sts?|ts?)?|'.		# -s, -sk, -skt, -st, -sts, -t, -ts
		         '(l)(s|skt?|ks?|sts?|ts?)?|'.		# -s, -sk, -skt, -k, -ks, -st, -sts, -t, -ts
		         '(g|j)(st?|(t|d)s?)?|'.			# -s, -t, -d, -ts, -ds, -st
		         '(rj|p)(t?s?|skt?)?|'.			# -t, -ts, -s, -sk, -skt
		         '(l\+)(s(\+)?(kt?)?|t\+)?|'.		# -s, -sk, -skt, -s+, -s+k, -s+kt, -t+
		         '(n\+)(s(\+)?|t\+s?)?|'.		# -s, -s+, -t+
		         '(t\+|d\+)(s\+|s\+kt?)?|'.		# -s+, -s+k, -s+kt
		         '(V(d|t)|ft|rl\+|g(d|t)|j(d|f|k|t)|ld)s?|'.	# -s
		         '(l(k|n|t)|m(b|d|f|t)|n(s\+|t)|rg)s?|'.		# -s
		         '(s(m|p|t))s?|'.			# -s
		         '(rb)(s|t)?|'.			# -s, -t
		         '(s\+)(sk?|kt?|t\+)?|'.		# -s, -sk, -k, -kt, -t+
		         '(l(p|f)|mp|r(f|k|p))t?s?|'.		# -ts, -t, -s
		         '(nK|k)(s?t)?s?|'.			# -s, -st, -sts, -t, -ts
		         '(nK)(d|kt?|n)?s?|'.			# -d, -kt, -n, -ds, -ks, -k, -kts, -ns
		         '(s)(k(s|t)?)?|'.			# -k, -kt, -ks
		         '(mn|lj|nj|rj|rm|mj|lV|rV|V)(d|t)?s?)$';	# -d, -t, -ds, -ts, -s

$SWE_DERIV = '(ring(en|ar(na)?)?s?$|rig(a|e|t)?$|arta(de?s?|t)?$|ls(a|e|t)?$|'.
	     'verk(en|et)?s?$|aktig(e|a|t)?$|artig(e|a|t)?$|skap(et|en)?s?$|het(en|er)?s?$|'.
	     'sam(me|ma)?s?$|halt(ig(a|e)?)?$|^all|mssig(t|a|e)?$|sint(a|e)?$|'.
	     '^bar|bar(t|a|e)$|gd(e|a)?$|vis(e|a|t)?$|enlig(t)?$|vr(d(e|a)?|t)$|'.
	     'makar(en?|na)?s?$|bo(n|r(na)?)?s?$|'.
	     'sinnig(t|(e|a)?s?)$|sidig(a|e|t)?$)';
	     
1;	# return 1 to parser-file