#!perl -w

$printlogg = 0;
open LOGG, ">logg.txt" if $printlogg;

$phonemedelimiter = 0;                                     # (1/0) 1 = stter in mellanslag mellan fonemen
$stress_toneme_sylls = 1;                                  # (1/0) 1 = behller eventuella tryck-, tonem- och stavelsemarkeringar

$stress = "\"|\%|";
$err = "r|s`|d`|t`|n`|l`|n`=|l`=";
$retr = "s`|d`|t`|n`|l`|n`=|l`=";
$S_right_ctxt = "p|m|n|v|t|k";

%norwegian_lh2sampa_conversion_table = qw(
E&U0	E*u0
E+&Y	9*Y
E&I	{*I
A&I	A*I
O&I	O*Y
n+%\)	n`=
l+%\)	l`=
t&S	tS
n%\)	n=
l%\)	l=
u0:	}:
e+:	2:
A:	A:
e:	e:
i:	i:
u:	u:
y:	y:
o:	o:
@:	{:
U0	u0
E+	9
d+	d`
t+	t`
n+	n`
l+	l`
nK	N
r6	r
2'	""
'2	%
A	A
I	I
U	U
Y	Y
O	O
E	E
b	b
d	d
g	g
p	p
t	t
k	k
f	f
s	s
S	S
V	v
C	C
m	m
n	n
l	l
h	h
j	j
'	"
.	$
_	_
"	
	
);

# Start temp idnrs as three digit numbers
my $id = 1000;

# Compute temporary 'x-forms' from conversion table hash
foreach  (keys %norwegian_lh2sampa_conversion_table) {   
	my ($phoneme,$to) = ($_,$norwegian_lh2sampa_conversion_table{$_});
	my $plength = length($phoneme);
	$phoneme =~ s/([\+\&\~\'\"\.\_\\\{\}\)])/\\$1/g;     # Insert backslashes to avoid problems in regexp
	my $xform = "x$id";
#	print LOGG "$phoneme\t$xform\t$plength\n";
	# assumes max length of 4 characters in from-table
	$nor_lh2x_4{$phoneme} = $xform if $plength == 4;       # NB: global
	$nor_lh2x_3{$phoneme} = $xform if $plength == 3;       # NB: global
	$nor_lh2x_2{$phoneme} = $xform if $plength == 2;       # NB: global
	$nor_lh2x_1{$phoneme} = $xform if $plength == 1;       # NB: global
	$nor_x2sampa{$xform} = $to;                            # NB: global
	$id++;
} # end foreach

# Top level sub to be called from other script. Must be called with trk AND ortography. Returns a comment if need exists
sub nor_lh2sampa {
	my $comment;
	my $trk = shift;
	my $ort = shift;

	print LOGG "ORTHOGRAPHY:\t\t\t$ort\n" if $printlogg;
	print LOGG "BEGINNING OF TOP LEVEL SUB:\t$trk\n" if $printlogg;

	my ($k4,$k3,$k2,$k1,$f);
	foreach $k4 (keys %nor_lh2x_4) {$trk =~ s/$k4/$nor_lh2x_4{$k4}/g}
	foreach $k3 (keys %nor_lh2x_3) {$trk =~ s/$k3/$nor_lh2x_3{$k3}/g}
	foreach $k2 (keys %nor_lh2x_2) {$trk =~ s/$k2/$nor_lh2x_2{$k2}/g}
	foreach $k1 (keys %nor_lh2x_1) {$trk =~ s/$k1/$nor_lh2x_1{$k1}/g}		
	foreach $f (keys %nor_x2sampa) {$trk =~ s/$f/$nor_x2sampa{$f} /g}
	print LOGG "AFTER MAIN CONVERSION:\t\t$trk\n" if $printlogg;

	$trk =~ s/[\"\%\$\*]+//g unless $stress_toneme_sylls;
	$trk =~ s/\s//g unless $phonemedelimiter;
	print LOGG "AFTER SUBSTITUTIONS:\t\t$trk\n" if $printlogg;

	$trk = &fix_S($trk,$ort);
	print LOGG "AFTER S FIX:\t\t\t$trk\n" if $printlogg;

	$trk = &fix_short_E($trk,$ort);
	print LOGG "AFTER E FIX:\t\t\t$trk\n" if $printlogg;

	print LOGG "END OF TOP LEVEL SUB:\t\t$trk\n\n" if $printlogg;
	return ($trk);
}

# Subs to fix other things after conversion table stuff is done. Are called from toplevel sub.
sub fix_short_E {
	my $trk = shift;
	my $ort = shift;
	my $syll = "";
	my $count = 0;

	#special rules for er- words
	$trk =~ s/^Er\$/\{r\$/;	#endrer erfare men ikke erindre (trykksvak frstestavelse som beg. p er)
	$trk =~ s/^E\$($stress)*($retr)/\{\$$1$2/;	#endrer erstatte (trykksvak frstestavelse som beg. p er)

	$trk =~ s/\$Er\$/\$\{r\$/ if $ort =~ /\+er/;
	$trk =~ s/\$E\$($stress)*($retr)/\$\{\$$1$2/ if $ort =~ /\+er/;	#skadeerstatning

	my @sylls = split /\$/, $trk;
	foreach $syll (@sylls) {
		if (($syll =~ /^E/) && ($count == 0)) {
			$count++;
			next;
		}
		
		elsif ($syll =~ /E\*/) {
			$count++;
			next;
		}

		else {
			$syll =~ s/E/\@/ if ($syll !~ /^$stress/);
			$count++;
		}	
	}
	$trk = join "\$", @sylls;
	$trk =~ s/E_/\@_/g;
	$trk =~ s/(.+)E(\$)?($err)/$1\{$2$3/;

	return $trk;
} #end sub

sub fix_S {
	my $trk = shift;
	my $ort = shift;
	my $syll = "";
	my $ortflag = 0;

	$ort =~ s/\+s\+/s+/g;

	if (($ort =~ /r(d|l|t)?s/) && ($trk =~ /S/)) {		#ingen endring dersom ikke rs i ortografi, og heller ikke i farskap
		if (($trk =~ /^(\")*S/) and ($trk !~ /S.*S/)) {	#ingen endring dersom S fins i begynnelsen og ingen andre steder
			return;
		}
		
		else {
			$trk =~ s/S$/s`/ if ($ort =~ /r(d|l|t)?s$/);	#endrer siste S i alle ord som slutter p rs (fars),
			$trk =~ s/S/s`/ if (($ort =~ /r(d|l|t)?s\+/) && ($trk !~ /^(\")*S/));	#og tar sjansen p  endre frste gang i dekomponerte ord som ikke begynner p S i trk
		}

		my @sylls = split /\$/, $trk;
		foreach $syll (@sylls) {
			$syll =~ s/S/s`/ if (($syll =~ /^[\"\%]*S$S_right_ctxt/) and ($syll !~ /Sn=/));	#endrer forspill, forsvar, sprsml, etc
			$syll =~ s/S/s`/ if (($syll =~ /^[\"\%]*Sl/) and ($ort =~ /rsl/) and ($trk !~ /S.*S/));	#endrer forslag
			$syll =~ s/($err)S$/$1s`/;		#endrer farts, Karls, ferds, verns, etc
		}

		$trk = join "\$", @sylls;

		if ($trk !~ /S.*S/) {
			$trk =~ s/S/s`/;			#kan trygt endre hvis kun n S, og rs fins i ortografien (I hope)
		}
	}
	
	$trk =~ s/^($stress)?s`/$1S/;
	$trk =~ s/\$($stress)s`u:(\$)?n/\$$1Su:$2n/;
#	$trk =~ s//;
#sI$tu0$A$%s`u:$nn=
#$sI$tu0$A$%s`u:$n@r
	
	return $trk;
}

1;

__END__
