#!perl -w

# Script som r tnkt att generera ortografier fr naturliga tal fr
# svl bokml som riksml som nynorska, samt i svl gammal som ny 'tellemte'
# OBS att alla kombinationer av ml och tellemte inte ska frekomma. (riksml har bara gammal tellemte)
# OBS att jag ej vet det riktiga ortografiska sttet vid framfrstllda ental
# (heter det verkligen 'en hundre og niogtyve'? eller 'enhundreni og tyve'?)
# OBS att alternativ inte r fixade n (tex ein/eitt, tre/tri) samt ngra andra upprensningssaker.
# FIXED (GA)
# Underlag r Annes tabell samt en vvsida:
#               http://www.riksmalsforbundet.no/ordskiftet_0199.html
# /Lasse

## -------------------
## Parse command line (or hardcode arguments)
## -------------------

# set command line mode
$command_line = 1;

$usage = "

USAGE:
perl $0 <low_number-high_number> <bm|rm|nn> <onesfirst|tensfirst> > outfile
";

# get args from command line...
if ($command_line) {
	die "Wrong number of arguments $usage" if $#ARGV != 2;
	die "Wrong format in first argument 
(must be a range between 0-999999999999)\n$usage" if $ARGV[0] !~ /^\d+\-\d+$/;
	die "Wrong format in secon argument (must be 'bm'|'rm'|'nn'  for bokmal,riksmal or nynorsk)\n$usage" if $ARGV[1] !~ /^bm|rm|nn$/;
	die "Wrong format for third argument (must be 'onesfirst'|'tensfirst')\n$usage" if $ARGV[2] !~ /^onesfirst|tensfirst$/;
	($s,$maal,$rekke) = @ARGV;
}
# or hardcode them here
else {($s,$maal,$rekke) = qw(567-583 bm tensfirst)}

($low,$high) = split /\-/,$s;

## ----------
## Main loop
## ----------

# get the appropriate hashes
&initialize($maal);
foreach ($low..$high) {
	@s = &normalize($_);
	$out = &compute_number(@s);
#	print "$_ \($maal $rekke\)\t$out\n";
	print "$_\t$out\n";
}

#&initialize($maal);
#@s = &normalize($s);
#$out = &compute_number(@s);
#print "$s \($maal $rekke\)\t$out\n";

## -----------------
## End of main loop
## -----------------







## -------------------
## Normalize argument
## to fixed size with
## leading zeros and
## make it a list of
## 12 digits (billions)
## -------------------
sub normalize {
	my @s = split //,shift;
	foreach (1..12-scalar(@s)) {unshift @s, "0"}
	return @s;
}

## --------------------------------------
## Initialize some global number strings
## --------------------------------------
sub initialize {
	my $maal = shift;
	
	# riksml
	%one = qw(0	null 1 en 2 to 3 tre 4 fire 5 fem 6 seks 7 syv 8 tte 9 ni) if $maal eq "rm";
	%ten_low = qw(0 ti 1 elleve 2 tolv 3 tretten 4 fjorten 5 femten 6 seksten 7 sytten 8 atten 9 nitten) if $maal eq "rm";
	%ten = qw(2 tyve 3 tredve 4 frti 5 femti 6 seksti 7 sytti 8 tti 9 nitti) if $maal eq "rm";
	
	#bokml
	%one = qw(0	null 1 en 2 to 3 tre 4 fire 5 fem 6 seks 7 sju 8 tte 9 ni) if $maal eq "bm";
	%ten_low = qw(0 ti 1 elleve 2 tolv 3 tretten 4 fjorten 5 femten 6 seksten 7 sytten 8 atten 9 nitten) if $maal eq "bm";
	%ten = qw(2 tjue 3 tretti 4 frti 5 femti 6 seksti 7 sytti 8 tti 9 nitti) if $maal eq "bm";
	
	#nynorsk
	%one = qw(0	null 1 ein 2 to 3 tre 4 fire 5 fem 6 seks 7 sju 8 tte 9 ni) if $maal eq "nn";
	%ten_low = qw(0 ti 1 elleve 2 tolv 3 tretten 4 fjorten 5 femten 6 seksten 7 sytten 8 atten 9 nitten) if $maal eq "nn";
	%ten = qw(2 tjue 3 tretti 4 frti 5 femti 6 seksti 7 sytti 8 tti 9 nitti) if $maal eq "nn";
}


# ----------------------------
# Computing a number is
# cleaning up computed billion
# ----------------------------
sub compute_number {
	return cleanup(compute_billions(@_));
}

# -------------------------------------
# Computing a billion is concatenating
# a computed hundred string, its glue
# phrase and computing a million
# -------------------------------------
sub compute_billions {
	if ($maal eq "nn") {
		return compute_hundreds(@_[0..2])." milliardar ".compute_millions(@_[3..11]);
	}
	else {return compute_hundreds(@_[0..2])." milliarder ".compute_millions(@_[3..11])};
}

# -------------------------------------
# Computing a million is concatenating
# a computed hundred string, its glue
# phrase and computing a thousand
# -------------------------------------
sub compute_millions {
	if ($maal eq "nn") {
		return compute_hundreds(@_[0..2])." millionar ".compute_thousands(@_[3..8]);
	}
	else {return compute_hundreds(@_[0..2])." millioner ".compute_thousands(@_[3..8])};
}

# -------------------------------------
# Computing a thousand is concatenating
# a computed hundred string, its glue
# phrase and computing another computed
# hundred string
# -------------------------------------
sub compute_thousands {
	return compute_hundreds(@_[0..2])." tusen ".compute_hundreds(@_[3..5]);
}

# -------------------------------------
# Computing a hundred is concatenating
# a looked-up one string, its glue
# phrase and computing a ten
# -------------------------------------
sub compute_hundreds {
	return $one{$_[0]}." hundre og ".compute_tens(@_[1,2]) if $_[0] > 0;
	return compute_tens(@_[1,2]);	
}

sub compute_tens {
	my ($ten,$one) = @_;
	my $ret;
	$ret = $one{$one}."\+og\+".$ten{$ten} if (($ten > 1) && ($one != 0));
	$ret = $one{$one}."og".$ten{$ten} if (($ten > 1) && ($one == 0));
#LW	$ret = $one{$one}."og".$ten{$ten} if $ten > 1;
	$ret = $one{$one} if $ten < 1;
	$ret = $ten_low{$one} if $ten eq "1";
#LW	if ($ret =~ /^(.*)og(.*)$/ and $rekke eq 'tensfirst') {$ret = $2.$1}
	if ($ret =~ /^(.*)\+og\+(.*)$/ and $rekke eq 'tensfirst') {$ret = $2."\+".$1}
	return $ret;
}

sub cleanup {
	my $ret = shift;
	$ret =~ s/og null\s?//g;
	$ret =~ s/null \S+ //g;
	$ret =~ s/null\+og//g;		#inserted \+ (GA)
	$ret =~ s/nullog//g;
	$ret =~ s/\s+null$//;
	$ret =~ s/null$//;
	$ret =~ s/( og)(\S)/$1 $2/g;
	$ret =~ s/^$/null/;
	$ret =~ s/($one{'1'} \S+)(er|ar)/$1/go;

#Added GA:
	$ret =~ s/( |^)ein (hundre|tusen)/$1eitt $2/g;
	$ret =~ s/( |^)en (hundre|tusen)/$1ett $2/g;
	$ret =~ s/ /_/g;
	$ret =~ s/(_|\+)$//g;
	$ret =~ s/^(_|\+)//g;
	$ret =~ s/_\+/_/g;

	return $ret;
}
